#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections, time
from stack import Stack

class Stacks:
    """Atualmente executando pedidos."""

    class Error(Exception):
        """Exceção levantada quando algo deu errado."""
        pass

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, timeout, tries):
        """Cria um novo conjunto de solicitações vazio."""

        # Verifica se o timeout é válido
        # if timeout is not None and timeout < 0:
        #     raise Stacks.Error("o tempo limite deve ser não-negativo ou None")

        # Verifica se o número de tentativas é válido
        # if tries is not None and tries <= 0:
        #     raise Stacks.Error("as tentativas devem ser positivas ou None")

        # Cria as pilhas
        self.list_waiting = collections.deque()
        self.list_processing = collections.deque()

        # Lista temporária para remover requisições da pilha de processamento
        self.temp_stack_requests = {}
        self.timeout = timeout
        self.tries = tries

    # --------------------------------
    # add
    # --------------------------------
    def add(self, message):
        """Crie um novo objeto de solicitação e faça com que ele entre no sistema."""
        self.list_waiting.append(Stack(message, self.timeout, self.tries))

    # --------------------------------
    # process
    # --------------------------------
    def process(self, stack):
        """Defina um novo parâmetro de solicitação antes de enviá-lo para um worker."""

        # Adiciona a requisição na lista de processamento
        self.list_processing.append(stack)

        # Registra a requisição na fila temporária
        self.temp_stack_requests[stack.id] = stack

        stack.make_request()
        return stack

    # --------------------------------
    # has_waiting
    # --------------------------------
    def has_waiting(self):
        # Verifica se há algum pedido aguardando um worker
        return len(self.list_waiting) > 0

    # --------------------------------
    # get_stack
    # --------------------------------
    def get_stack(self):
        # Obtém o primeiro pedido na fila
        if self.has_waiting():
            # Remove requisição da lista de espera
            stack = self.list_waiting.popleft()

            # Adiciona processo na lista de processamento
            # Retorna o processo na requisição
            return self.process(stack)
        raise Stacks.Error("internal error")

    # --------------------------------
    # check_for_timeouts
    # --------------------------------
    def check_for_timeouts(self):
        """Verifica se algum pedido expirou. Se esse é o caso,
        coloca na fila novamente a menos que o número máximo de tentativas
        tenha sido alcançado. Retorna uma lista de pedidos expirados."""
        expired = []
        while self.list_processing and self.list_processing[0].timed_out():
            request = self.list_processing.popleft()
            del self.temp_stack_requests[request.id]
            if request.expired():
                expired.append(request)
            else:
                self.list_waiting.append(request)
        return expired

    # --------------------------------
    # remove
    # --------------------------------
    def remove(self, result):
        """Dada uma resposta, remove a solicitação correspondente da fila.
        Retornar a resposta para enviar se o pedido foi encontrado no sistema,
        nenhum caso (resposta tardia, por exemplo)."""
        id, content = result[0], result[1:]

        if id in self.temp_stack_requests:
            request = self.temp_stack_requests[id]
            self.list_processing.remove(request)
            del self.temp_stack_requests[id]

            return request.path + content

    # --------------------------------
    # next_timeout_ms
    # --------------------------------
    def next_timeout_ms(self):
        """Retorna o número de milisegundos até o próximo evento de tempo limite,
           ou None se não houver nenhum."""
        if self.list_processing:
            return 1000 * max(self.list_processing[0].deadline - time.time(), 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pprint import pprint


# --------------------------------
# Response
# --------------------------------
# A classe Response gerencia o retorno de consulta no servidor de configuração
class Response:
    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, service_id, data, status):
        self.__ready = False
        self.__data = None
        self.__id = service_id

        self.status = status
        self.__check(service_id, data)

    # --------------------------------
    # data
    # --------------------------------
    # Retorna os dados de configuração
    def data(self):
        return self.__data

    # --------------------------------
    # ready
    # --------------------------------
    # Status indica se os dados estão prontos
    def ready(self):
        return self.__ready

    # --------------------------------
    # show
    # --------------------------------
    # Imprime propriedade/valor do objeto
    def show(self, message=None):
        if message:
            print message, self.__id, ":::------->>>"
        pprint(self.__data)

    # --------------------------------
    # __check
    # --------------------------------
    # Verifica se os dados retornados são válidos
    '''
    :param service_id: Identificador do serviço
    :param data: Dados retornados do serviço de configuração
    :return: void
    '''

    def __check(self, service_id, data):
        # Verifica se os dados é do tipo lista ou dicionário e se possui valor atribuído
        if ((type(data) == list) and (len(data) > 0)) or ((type(data) == dict) and (len(data.keys()) > 0)):
            self.__data = data
            self.__ready = True
        else:
            self.status.error("RESPONSE", None, ["Configuracao de servico nao encontrada", service_id])

        #self.show("RESPONSE")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests
from data_response import DataResponse

# --------------------------------
# Connection
# --------------------------------
'''
A classe Connection realiza conexão com servidor de serviço API
'''

class Connection:

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, status):
        self.status = status

    # --------------------------------
    # load_config
    # --------------------------------
    '''
    Conecta o servidor de serviços API e recupera documentos
    '''
    def load_config(self, service_id):
        print ('SERVICE_ID =======>', service_id)
        # url = 'http://10.1.0.90:9500/console/config'
        url = 'http://localhost:9500/console/config'
        params = dict(
            service=service_id
        )
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Accept': 'application/json',
            'devdoo-api-key': '286c42a2b9dabb536c87b1a88a6842117bfb37ab',
            'devdoo-app-id': '507f191e810c19729de860ea',
            'devdoo-credentials': 'NTA3ZjE5MWU4MTBjMTk3MjlkZTg2MGVhOjhmNGY5ZjJkY2JhZjliZDhlZTllNGQxYTJjYjk3MWEwNDA4NDE1N2I='
        }
        data_response = requests.get(url, params=params, headers=headers)

        if type(data_response) == requests.models.Response:
            return DataResponse(data_response.json(), self.status)
        else:
            self.status.error("ERROR_CONNECTION", None, ["Falha na conexao com servidor de servicos API", service_id])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from actions import Actions
from service import Service
from status import Status
from yml import Yml
from pprint import pprint

# --------------------------------
# DevdooService
# --------------------------------
'''
A classe DevdooService prepara todas as configurações database dos serviços API
'''
# TODO:: Enviar para o servidor console a informação de falha de configuração
class DevdooService(object):

    # --------------------------------
    # Error
    # --------------------------------
    # TODO:: Remover classe de error
    class Error(Exception):
        pass

    # --------------------------------
    # __init__
    # --------------------------------
    '''
    Inicializa a classe
    
    :return: void
    '''
    def __init__(self):
        super(DevdooService, self).__init__()

        # Variaveis default
        self.actions = None
        self.id = None
        self.service = None
        self.zmq_socket_database = None
        self.zmq_socket_rest = None

        # Mensagens de alerta/erros
        self.status = Status()

        # Verifica se o arquivo de configuração foi inicializado corretamente
        if self.__has_config():
            self.__init()
        else:
            self.status.error("INVALID_SERVICE", None, ["FALHA NA INICIALIZACAO DO SERVICO", "DEVDOO-SERVICE"])
            self.status.to_print()

    # --------------------------------
    # __has_config
    # --------------------------------
    '''
    Obtem configurações do servico
    
    :return: boolean
    '''
    def __has_config(self):
        # Recupera o identificador do servico
        yml = Yml(self.status)
        self.id = yml.id()

        # Verifica se existe um identificador do servico
        if self.status.ready() and self.id:
            self.service = Service(self.id, self.status, True)

        # Retorna o status do serviço de API
        return self.status.ready() and self.service and self.service.ready()

    # --------------------------------
    # __init
    # --------------------------------
    '''
    Inicializa o serviço
    
    :return: void
    '''
    def __init(self):
        # Cria contexto ZMQ
        context = zmq.Context()

        # Adaptador de conexão broker secundário
        self.zmq_socket_database = context.socket(zmq.REQ)
        self.zmq_socket_database.connect(self.service.database_connect_frontend())

        # Adaptador de conexão broker primário
        self.zmq_socket_rest = context.socket(zmq.REP)
        self.zmq_socket_rest.connect(self.service.broker_connect_backend())

        print "Frontend:", self.service.database_connect_frontend()
        print "Backend:", self.service.broker_connect_backend()

        # Executa o loop da operação
        self.run()

    # --------------------------------
    # operation
    # --------------------------------
    '''
    Executa a ação do Serviço
    
    :return: void
    '''
    def operation(self):
        # Recebe mensagem do servidor cliente
        message = self.zmq_socket_rest.recv_multipart()
        print "RECV <<<-->>>", message

        # Inicia o Serviço
        self.actions = Actions(message, self.service)

        # Verifica se o servico está pronto
        if self.actions.ready():
            # Envia mensagem para o servidor de base de dados
            self.zmq_socket_database.send_multipart([self.actions.send_database()])
            # print ("SOCKET VALIDO ENVIOU ADIANTE ----->>>>", self.actions.send_database())

            # Recebe mensagem do servidor de base de dados
            # Envia mensagem de retorno para o cliente
            # print "XXXX--->", self.service.broker_connect_backend()

            rec_msg = self.zmq_socket_database.recv()
            # print ("RECV DATABASE", rec_msg)
            self.zmq_socket_rest.send(rec_msg)
            print ("SOCKET VALIDO VOLTOU <<<<------")

        # Em caso de falha
        else:
            # Nao processa a operação e envia mensagem de retorno para o cliente
            self.zmq_socket_rest.send(self.actions.send_error())
            print ("SOCKET FALHOU E VOLTOU <<<<------")

    # --------------------------------
    # run
    # --------------------------------
    #
    '''
    Executa a operação em um loop infinito

    :return: void
    '''
    # TODO:: Implementar ping de verificação de disponibilidade do serviço
    def run(self):
        while True:
            try:
                self.operation()
            except DevdooService.Error:
                # except Exception as inst:
                #    print "API::RUN::ERROR", inst.args
                self.status.error("SERVICE_FAILURE", None, ["FALHA NA EXECUCAO DO SERVICO", "DEVDOO-SERVICE"])
                self.status.to_print()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pprint import pprint

# --------------------------------
# DataResponse
# --------------------------------
'''
A classe Response gerencia o retorno de consulta no servidor de configuração
'''

class DataResponse:

    # --------------------------------
    # __init__
    # --------------------------------
    '''
    Inicializa a classe

    :param data: Documento retornado do serviço de API
    :param status: Gerenciador de erros
    '''
    def __init__(self, data, status):
        # Variaveis privadas
        self.__values = None

        # Variaveis parametros
        self.status = status

        self.__config(data)

    # --------------------------------
    # ready
    # --------------------------------
    '''
    Status indica se está pronto
    
    :return: void
    '''
    def ready(self):
        return self.status.ready()

    # --------------------------------
    # show
    # --------------------------------
    '''
    Imprime propriedade/valor do objeto
    
    :param title: Título identificador
    :return: void
    '''
    def show(self, title=None):
        if title is not None:
            print title, ":::------->>>"
        pprint(self.__values)

    # --------------------------------
    # values
    # --------------------------------
    '''
    Retorna os valores obtidos no servidor de serviços API
    
    :return: void
    '''
    def values(self):
        return self.__values

    # --------------------------------
    # __config
    # --------------------------------
    '''
    Verifica se os dados retornados são válidos
    
    :param data: Dados retornados do servidor de serviços API
    :return: void
    '''
    def __config(self, data):
        # Verifica se os dados é do tipo lista ou dicionário e se possui valor atribuído
        if ((type(data) == list) and (len(data) > 0)) or ((type(data) == dict) and (len(data.keys()) > 0)):
            self.__values = data
        else:
            self.status.error("DATA_RESPONSE", None,
                              ["Falha da configuracao de dados retornados do servidor de servicos API"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import unicodedata
from bson import Int64
from bson.timestamp import Timestamp
from bson.objectid import ObjectId


# TODO:: Testar possivel falha em str(inst.args)
class Check:
    def __init__(self):
        pass

    # --------------------------------
    # is_array
    # --------------------------------
    # Verifica se é um array
    # Retorna boolean
    #
    @staticmethod
    def is_array(value):
        return type(value) == list

    # --------------------------------
    # is_boolean
    # --------------------------------
    # Verifica se é boleano
    # Retorna boolean
    #
    @staticmethod
    def is_boolean(value):
        return type(value) == bool

    # --------------------------------
    # is_date
    # --------------------------------
    # Verifica se é uma data
    # Retorna boolean
    #
    @staticmethod
    def is_date(value):
        status = False
        if (type(value) == dict) and ("$date" in value.keys()):
            status = (len(value["$date"]) == 24)
        return status

    # --------------------------------
    # is_decimal
    # --------------------------------
    # Verifica se é decimal
    # Retorna boolean
    #
    @staticmethod
    def is_decimal(value):
        status = False
        if (type(value) == dict) and ("$numberDecimal" in value.keys()) and re.match("^-?[0-9]+\.?[0-9]*$", value["$numberDecimal"]):
            status = (len(value["$numberDecimal"]) > 0) and (len(value["$numberDecimal"]) <= 34)
        return status

    # --------------------------------
    # is_default
    # --------------------------------
    @staticmethod
    def is_default(item_scheme):
        return 'default' in item_scheme.keys()

    # --------------------------------
    # is_empty
    # --------------------------------
    # Verifica se o valor do campo não está vazio ou é do tipo js null
    @staticmethod
    def is_empty(value):
        try:
            if type(value) == unicode:
                value = unicodedata.normalize('NFKD', value).encode('utf-8', 'ignore')
            else:
                value = str(value)
        except Exception:
            value = ''

        # Verifica se o campo é diferente de string e se não está vazio
        return (len(value) == 0) or (value == '')

    # --------------------------------
    # is_number
    # --------------------------------
    # Verifica se é número
    # Retorna boolean
    #
    @staticmethod
    def is_number(value):
        return (type(value) == Int64) and (value >= -9223372036854775808) and (value <= 9223372036854775807)

    # --------------------------------
    # is_object
    # --------------------------------
    # Verifica se é objeto
    # Retorna boolean
    # TODO:: Implementar validação recursiva para valores numéricos
    #
    @staticmethod
    def is_object(value):
        return (type(value) == dict) and ("error" not in value.keys())

    # --------------------------------
    # is_object_id
    # --------------------------------
    # Verifica se é objectId
    # Retorna boolean
    #
    @staticmethod
    def is_object_id(value):
        return type(value) == ObjectId

    # --------------------------------
    # is_scheme
    # --------------------------------
    @staticmethod
    def is_scheme(scheme):
        if type(scheme) == dict:
            list_scheme = ["$currentDate", "$inc", "$min", "$max", "$mul", "$rename", "$set", "$setOnInsert", "$unset"]
            for key in scheme.keys():
                if key in list_scheme:
                    return True
        return False

    # --------------------------------
    # is_string
    # --------------------------------
    # Verifica se é string
    # Retorna boolean
    #
    @staticmethod
    def is_string(value):
        return (type(value) == unicode) or (type(value) == str)

    # --------------------------------
    # is_timestamp
    # --------------------------------
    # Verifica se é timestamp
    # Retorna boolean
    #
    @staticmethod
    def is_timestamp(value):
        return type(value) == Timestamp

    # --------------------------------
    # value_string
    # --------------------------------
    @staticmethod
    def value_string(field, value, status):
        # Verifica se recebeu configuração de endpoint
        if (value is not None) and (Check.is_empty(value) is False):
            return value
        else:
            status.error("INVALID_CONFIG", None, [field])

    # --------------------------------
    # value_dict
    # --------------------------------
    @staticmethod
    def value_dict(field, value, status):
        # Verifica se recebeu configuração de endpoint
        if (type(value) == dict) and (field in value):
            return value[field]
        else:
            status.error("INVALID_CONFIG", None, [field])

    # --------------------------------
    # value_list
    # --------------------------------
    @staticmethod
    def value_list(field, value, status):
        # Verifica se recebeu configuração de services
        if (value is not None) and (type(value) == list) and (len(value) > 0):
            return value
        else:
            status.error("INVALID_CONFIG", None, [field])




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from check import Check
from convert import Convert
from bson.json_util import dumps
from pprint import pprint

# TODO: Refatorar classe completa Pack
# TODO: Alterar o nome da classe para pack
class Pack:

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, message, status):
        self.__ready = False
        self.message = message
        self.status = status
        self.pack = None

        self.action = None  # Ação a ser executada (get, post, put, delete?)
        self.active_port = None  # Identificador da porta do serviço ativo
        self.alerts = {
            "error": None,  # Mensagens de erro para o desenvolvedor corrigir a aplicação ou apresentar ao usuário
            "log": None,  # Mensagens de log para o desenvolvedor corrigir sua aplicação
            "warn": None,  # Mensagens de warn para o desenvolvedor apresentar ao usuário
            "info": None,  # Mensagens de info para o desenvolvedor apresentar ao usuário
        }
        self.app_id = None  # Id da aplicação
        self.api_key = None  # Chave de acesso da api
        self.body = dict()  # Todos os dados que precisam ser transportados entre os serviços
        self.id = None  # Id da conexão, gerado no REST
        self.length_in = 0  # Tamanho do pacote de entrada
        self.length_out = 0  # Tamanho do pacote de saída
        self.service = None  # Identificador do serviço
        self.service_id = None  # Id do serviço a ser executado
        self.database_id = None  # Id numérico da base de dados
        self.open = True  # Indica se a conexão está aberta/fechada 0=close|1=open
        self.source = None  # Uri da api
        self.success = False  # Status boleano indicando sucesso ou falha na operação
        self.time = {
            "time_start": 0,  # Momento de inicio da conexão no REST
            "service": 0,  # Momento de inicio da operação no serviço
            "database": 0  # Momento de inicio da operação do serviço de banco
        }
        self.__decode()

    # --------------------------------
    # decode
    # --------------------------------
    def __decode(self):
        if (type(self.message) == list) and (len(self.message) == 1):
            self.message = self.message[0]
        #print "[[[[[[[SELF.MESSAGE]]]]]]]]", self.message
        # Verifica se a mensagem é do tipo lista e tem mais de três elementos e o terceiro elemento seja do tipo string
        if Check.is_string(self.message):
            # Converte o terceiro elemento da lista em object (dic)
            pack = Convert.to_object(self.message)

            # Verifica se é um objeto válido
            if Check.is_object(pack):
                # self.active_port = pack["active_port"] # TODO:: Remover active port
                self.action = pack["action"]
                self.alerts = {
                    "error": pack["alerts"]["error"],
                    "log": pack["alerts"]["log"],
                    "warn": pack["alerts"]["warn"],
                    "info": pack["alerts"]["info"]
                }
                self.app_id = pack["app_id"]
                self.api_key = pack["api_key"]
                self.body = pack["body"]
                self.id = pack["id"] # id da conexão (criado no connection - rest)
                self.length_in = sys.getsizeof(self.message)
                self.length_out = pack["length_out"]
                self.service = Convert.to_str(pack["service"])
                self.service_id = Convert.to_str(pack["service_id"])
                self.database_id = Convert.to_str(pack["database_id"])
                self.open = pack["open"]
                self.source = pack["source"]
                self.success = pack["success"]
                self.time = {
                    "time_start": pack["time"]["time_start"],
                    "service": pack["time"]["service"],
                    "database": pack["time"]["database"]
                }
                self.database = "db" + self.database_id + self.service
                self.token = "xvz1evFS4wEEPTGEFPHBog:L8qq9PZyRg6ieKGEKhZolGC0vJWLw8iEJ88DRdyOg"
                self.__ready = True
            else:
                self.status.error("CONFIGURATION_SERVICE", None, ["Arquivo de configuracao invalido"])
        else:
            self.status.error("CONFIGURATION_SERVICE", None, ["Arquivo de configuracao invalido"])

    # --------------------------------
    # is_valid
    # --------------------------------
    def is_valid(self):
        self.__check_pack()
        return self.ready()

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__ready and self.status.ready()

    # --------------------------------
    # send_error
    # --------------------------------
    # Montar mensagem de erro para ser retornada ao servidor rest
    # TODO: Preparar filtro para ser retornado ao cliente
    def send_error(self):
        message = {
            "action": self.action,
            "active_port": self.active_port,
            "alerts": {
                "error": self.status.to_list(),
                "log": None,
                "warn": None,
                "info": None,
            },
            "app_id": self.app_id,
            "api_key": self.api_key,
            "body": {
                "success": False,
                "message": "Falha de interação com microserviço",
                "elapsed_time": {"response": "17.000", "server": "7.000"},
                "query": {},
            },
            "id": self.id,
            "length_in": self.length_in,
            "length_out": None,
            "service": self.service,
            "service_id": self.service_id,
            "database_id": None,
            "open": False,
            "source": self.source,
            "success": False,
            "time": {
                "time_start": None,
                "service": None,
                "database": None
            }
        }
        return [dumps(message)]

    # --------------------------------
    # to_string
    # --------------------------------
    def to_string(self):
        if self.pack:
            return dumps(self.pack)
        return '{}'

    # --------------------------------
    # __check_pack
    # --------------------------------
    def __check_pack(self):
        # Verifica se mensagem é string
        if (type(self.message) == list) and len(self.message) >= 3:
            # Converte a mensagem recebida do servidor rest para objeto
            self.pack = Convert.to_object(self.message[2])

            # Verifica se o pacote de mensagem é do tipo objeto válido
            if Check.is_object(self.pack):
                # Verifica se o pacote de mensagem é válido
                self.__ready = self.__check_params(self.pack)

                if self.__ready is not True:
                    self.status.error("INVALID_PACKAGE", None, ["Pacote de comunicacao invalido"])
                    self.pack["alerts"] = {"error": self.status.to_list(), "log": None, "warn": None, "info": None}
                    self.pack["success"] = False
                    self.pack["status"] = 0

    # --------------------------------
    # __check_params
    # --------------------------------
    # Verifica se o pacote de comunicação é válido
    def __check_params(self, pack):
        errors = []

        # Verifica se no pacote existe o identificador do serviço
        if "action" not in pack.keys():
            errors.append("ACTION")

        if "alerts" not in pack.keys():
            errors.append("ALERTS")

        if "body" not in pack.keys():
            errors.append("BODY")

        if "length_in" not in pack.keys():
            errors.append("LENGTH_IN")

        if "length_out" not in pack.keys():
            errors.append("LENGTH_OUT")

        if "open" not in pack.keys():
            errors.append("OPEN")

        if "source" not in pack.keys():
            errors.append("SOURCE")

        if "success" not in pack.keys():
            errors.append("SUCCESS")

        if "time" not in pack.keys():
            errors.append("TIME")

        if "api_key" not in pack.keys():
            errors.append("API_KEY")

        if "id" not in pack.keys():
            errors.append("ID")

        if "service" not in pack.keys():
            errors.append("SERVICE")

        if "service_id" not in pack.keys():
            errors.append("SERVICE ID")

        if len(errors) > 0:
            self.status.error("INVALID_PACKAGE_PARAMS", None, ["Pacote de comunicacao nao tem:", (",".join(errors))])
            return False
        return True

    # --------------------------------
    # show
    # --------------------------------
    def show(self, message=None):
        if message:
            print message, ":::------->>>"
        pprint(self.__dict__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from broker import Broker
from status import Status
from yml import Yml
from stacks import Stacks

# --------------------------------
# DevdooBroker
# --------------------------------
'''
A classe DevdooBroker gerencia a conexão entre o rest e os serviços API
'''
# TODO:: Enviar para o servidor console a informação de falha de configuração
# TODO:: Enviar para o console mensagem de erro de serviço não disponível

class DevdooBroker(object):

    # --------------------------------
    # Error
    # --------------------------------
    # TODO:: Remover classe de error
    class Error(Exception):
        pass

    # --------------------------------
    # __init__
    # --------------------------------
    '''
    Inicializa a classe
    
    :return: void
    '''
    def __init__(self):
        super(DevdooBroker, self).__init__()

        # Variaveis default
        self.broker = None
        self.timeout = 2
        self.tries = 3
        self.zmq_socket_service = None
        self.zmq_socket_client = None
        self.id_zmq = None

        # Mensagens de alerta/erros
        self.status = Status()

        # Verifica se o arquivo de configuração foi inicializado corretamente
        if self.__has_config():
            # Inicializa o broker
            self.__init()
        else:
            self.status.error("INVALID_SERVICE", None, ["FALHA NA INICIALIZACAO DO SERVICO", "DEVDOO-BROKER"])
            self.status.to_print()

    # --------------------------------
    # __has_config
    # --------------------------------
    '''
    Obtem configurações do servico
    
    :return: void
    '''
    def __has_config(self):
        # Recupera o identificador do servico
        yml = Yml(self.status)
        self.id = yml.id()
        print ('SELF.ID ====>', self.id)

        # Verifica se existe um identificador de servico válido
        if self.status.ready() and self.id:
            self.broker = Broker(broker_id=self.id, status=self.status)

        # Retorna o status do broker
        return self.status.ready()

    # --------------------------------
    # __init
    # --------------------------------
    '''
    Inicializa o serviço de Broker
    
    :return: void
    '''
    def __init(self):
        # Obtem o contexto ZMQ
        self.context = zmq.Context()

        # Inicializa o poll ZMQ
        self.poller = zmq.Poller()

        # Cria objeto de conexão para receber requisições do REST
        self.zmq_socket_client = self.context.socket(zmq.ROUTER)

        # Configura a porta de entrada para receber requisições do REST
        self.zmq_socket_client.bind(self.broker.bind_frontend)

        # Registra na lista de sockets o socket de entrada
        self.poller.register(self.zmq_socket_client, zmq.POLLIN)

        # Cria um novo gerenciador de solicitações vazio
        self.stacks = Stacks(self.timeout, self.tries)

        self.status.show("DEVDOO_BROKER_INIT", [self.broker.id])

        # Executa o loop de operação
        self.run()

    # --------------------------------
    # operation
    # --------------------------------
    '''
    Permuta pacotes entre o frontend e o backend
    
    :return: void
    '''
    # TODO:: Verificar se existe um serviço para o service_id solicitado antes de tentar pegar na lista
    # TODO:: Verificar se o serviço está disponível antes de tentar fazer conexão
    # TODO:: Verificar se a mensagem foi corretamente convertida de string para dic antes de tentar pegar um elemento
    # TODO:: Procurar formas de desregistrar sockets que não obtiveram respostas dentro do tempo limite
    def operation(self):
        #SEGUNDA OPERACAO
        # Indica se há requisições em espera
        has_waiting = self.stacks.has_waiting()

        # Verifica se há requisições em espera
        if has_waiting:
            # Obtém o primeiro pedido da fila
            stack = self.stacks.get_stack()

            # Verifica se há um pedido aguardando
            if stack.ready():
                # Obtém o socket do serviço
                self.zmq_socket_service = self.broker.socket(stack)
                if self.zmq_socket_service.ready():
                    # Registra requisição no poller
                    # self.poller.register(self.zmq_socket_service.connect_backend, zmq.POLLIN)

                    # Obtém a primeira mensagem do cliente da fila de espera
                    message = stack.message

                    # Envia o pacote para o Worker
                    self.zmq_socket_service.connect_backend().send_multipart(message)
                    print ('ID ENVIADA AO BACKEND ====>', message[0])

                    # Recebe retorno do Worker
                    worker_result = self.zmq_socket_service.connect_backend().recv_multipart()
                    print ('ID RECEBIDA DO BACKEND =====>', worker_result[0])

                    # Prepara a mensagem recebida
                    content = self.stacks.remove(worker_result)

                    # Verifica se o conteúdo existe
                    if content is not None:
                        worker_result[0] = self.id_zmq

                        # Retorna a resposta ao cliente
                        self.zmq_socket_client.send_multipart(worker_result)
                        #self.poller.unregister(self.zmq_socket_service.connect_backend)

        # PRIMEIRA OPERACAO
        # Obtém o tempo limite da próxima requisição
        timeout_ms = self.stacks.next_timeout_ms()

        # Itera a lista de poller e recupera um socket
        for (socket, event) in self.poller.poll(timeout_ms):
            # Verifica se o socket obtido é do tipo cliente
            if socket == self.zmq_socket_client:
                # Recebe solicitação do client REST
                msg = self.zmq_socket_client.recv_multipart()
                print ('ID RECEBIDA DO CLIENTE ====>', msg[0])

                # Salva id recebido do ZMQ
                self.id_zmq = msg[0]

                # Cria um novo request com a mensagem do cliente e adiciona na lista de espera
                self.stacks.add(msg)

        # Retorna para o cliente a mensagem contendo informações dos erros encontrados
        for stack in self.stacks.check_for_timeouts():
            self.zmq_socket_client.send_multipart(stack.send_error())

        # TERCEIRA OPERAÇÂO
        # Verifica se há requisições em espera
        '''
        if has_waiting and self.zmq_socket_service.ready():
            # Remove a requisição do poller
            if self.zmq_socket_service and (self.zmq_socket_service.connect_backend in self.poller):
                self.poller.unregister(self.zmq_socket_service.connect_backend)
            else:
                self.status.error("INVALID_SERVICE", None,
                    ["XXX Falha na configuracao do zmq_socket_service.connect_backend ", self.broker.id])
        '''
        # Imprime mensagens de erro/status
        self.status.to_print()

    # --------------------------------
    # run
    # --------------------------------
    '''
    Executa a operação em um loop infinito
    
    :return: void
    '''
    def run(self):
        # Executa indefinidamente
        while True:
            try:
                self.operation()
            except DevdooBroker.Error:
                # except Exception as inst:
                # print "BROKER::RUN::ERROR", inst.args
                self.status.error("SERVICE_FAILURE", None, ["FALHA NA EXECUCAO DO SERVICO", "DEVDOO-BROKER"])
                self.status.to_print()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datarest import DataRest
from check import Check
from validate import Validate
from status import Status

# TODO:: Fazer revisão completa da classe para fechar
# TODO:: Mover todas as mensagens de erros para a classe status
# TODO:: Remover todos os prints
class Actions(object):

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, message, service):
        self.service = service
        self.status = Status()
        self.endpoint = None

        self.database = service.database
        self.datarest = DataRest(message, self.status)
        self.validate = Validate(self.status)

        self.__config()
        self.status.to_print()

    # --------------------------------
    # action
    # --------------------------------
    def __config(self):
        print "DATAREST_READY", self.datarest.ready()
        print "SERVICE_READY", self.service.ready()
        print "DATABASE_READY", self.database.ready()

        if self.datarest.ready(): # and self.service.ready() and self.database.ready():
            # Obtém a fonte e o método recebidos no pacote
            source = self.datarest.source()
            method = self.datarest.method()

            # Obtém endpoint do banco de dados
            self.endpoint = self.database.endpoint(method, source, self.status)

            # Verifica se o endereço da API existe
            if self.endpoint.ready():
                # Registra o tipo de retorno de dados que deverá executar
                self.datarest.result_type(self.endpoint.type)

                # if hasattr(self, self.endpoint.action) and self.datarest.validate_service(self.name, self.id):
                if hasattr(self, self.endpoint.action):
                    action = getattr(self, self.endpoint.action)
                    action(self.endpoint)
                else:
                    self.status.error("INVALID_ACTION", None, [source, self.service.name, self.endpoint.action, method])

            else:
                self.status.error("INVALID_ENDPOINT", None, [source, self.service.name, method])

        else:
            self.status.error("INVALID_SERVICE", None, ["NAO ESTA PRONTO PARA EXECUTAR O SERVICO", "PARAMETROS FALTANDO"])

    # --------------------------------
    # find
    # --------------------------------
    def find(self, endpoint):
        # Define o tipo de ação que será executada no servidor de base de dados
        self.datarest.action("find")

        # Recupera o identificador da coleção
        self.datarest.collection(endpoint.collection)

        # Registra campos que devem ser retornados
        self.datarest.fields(endpoint.fields(self.datarest.public_access()))
        self.datarest.limit(endpoint.max_limit)

    # --------------------------------
    # insert
    # --------------------------------
    # Prepara documento para ser inserido na base de dados
    def insert(self, endpoint):
        list_insert_data = []

        # Define o tipo de ação que será executada no servidor de base de dados
        self.datarest.action("insert")

        # Recupera o identificador da coleção
        self.datarest.collection(endpoint.collection)

        #self.database.show()

        # Prepara a lista de campos válidos que poderão ser inseridos na base de dados
        # Recebe a lista de erros em campos recebidos no body
        # Recebe a lista de erros em campos definido pelo desenvolvedor de microserviço
        list_scheme_clean = self.database.fields_insert(self.datarest.body(), endpoint)

        # Processa todos os campos registrados no schema para validar os valores e formatar
        for item_scheme_name in list_scheme_clean:
            # Recupera o esquema para validar e formatar
            item_scheme = list_scheme_clean[item_scheme_name]

            # Recupera o valor que deverá ser validado e formatado
            field_value = item_scheme["value"]

            # Valida o campo e valor
            # Recebe o nome do campo que será inserido na base de dados e depois retornado
            # Recebe o valor que deverá ser insrido na base de dados
            field_name, field_value = self.validate.field_insert(item_scheme_name, item_scheme, field_value, "custom_insert")

            # Verifica se o campo não está com erro de validação
            if field_value != '__error__' and field_value != '__ERROR__':
                # Registra o campo na lista de dados que deverá ser enviada para o servidor de base de dados
                if field_name != "id":
                    list_insert_data.append({"field": field_name, "value": field_value})

        # Prepara s lista de campos que será enviada para o servidor de base de dados
        # Prepara a lista de campos que devrá ser retornada do servidor depois de salvar na base de dados
        list_insert_data = self.__prepare_insert_data_fields(list_insert_data)

        # Registra os dados do documento
        self.datarest.data(list_insert_data)

        # Verificar se tem algo para gravar no banco, caso contrário, registra mensagem de error
        if len(list_insert_data) <= 0:
            self.status.error("NO_DATA_TO_INSERT", None, ['insert'])

    # --------------------------------
    # prepare_update_data
    # --------------------------------
    # Prepara lista de campos que podem ser atualizados no documento
    @staticmethod
    def prepare_update_data(list_data):
        result = dict()
        for item in list_data:
            # Registra o campo e valor na lista de campos válida do documento
            result[(item["field"])] = item["value"]
        return result

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.status.ready()

    # --------------------------------
    # remove
    # --------------------------------
    def remove(self, endpoint):
        # Define o tipo de ação que será executada no servidor de base de dados
        self.datarest.action("remove")

        # Recupera o identificador da coleção
        self.datarest.collection(endpoint.collection)

    # --------------------------------
    # send_database
    # --------------------------------
    def send_database(self):
        return self.datarest.send_database()

    # --------------------------------
    # send_error
    # --------------------------------
    def send_error(self):
        return self.datarest.send_service_error()

    # --------------------------------
    # update
    # --------------------------------
    def update(self, endpoint):
        # Validar configuração de schema para update, erro developer
        if Check.is_scheme(endpoint.scheme):
            # Define o tipo de ação que será executada no servidor de base de dados
            self.datarest.action("update")

            # Recupera o identificador da coleção
            self.datarest.collection(endpoint.collection)

            update_data = dict()
            fields = ["_id"]
            for item_scheme in endpoint.scheme:
                obj_item_scheme = self.update_config(endpoint.scheme[item_scheme], endpoint.fields(self.datarest.public_access()))

                if len(obj_item_scheme.keys()) > 0:
                    fields = fields + obj_item_scheme.keys()
                    update_data[item_scheme] = obj_item_scheme

            # Registra os dados do documento
            self.datarest.data(update_data)

            # Registra campos que devem ser retornados
            self.datarest.fields(fields)

        else:
            self.status.error("INVALID_SCHEME_UPDATE", None, [])

    # --------------------------------
    # update_config
    # --------------------------------
    def update_config(self, scheme, fields_default):
        list_update_data = []

        # Prepara a lista de campos válidos que poderão ser inseridos na base de dados
        # Recebe a lista de erros em campos recebidos no body
        # Recebe a lista de erros em campos definido pelo desenvolvedor de microserviço
        list_scheme_clean = self.database.fields_update(self.datarest.body(), scheme, fields_default)

        print "LIST_SCHEME_CLEAN", list_scheme_clean, self.datarest.body()

        # Processa todos os campos registrados no schema para validar os valores e formatar
        for item_scheme_name in list_scheme_clean:
            # Recupera o equema para validar e formatar
            item_scheme = list_scheme_clean[item_scheme_name]

            # Recupera o valor que deverá ser validado e formatado
            field_value = item_scheme["value"]

            # Valida o campo e valor
            # Recebe o nome do campo que será inserido na base de dados e depois retornado
            # Recebe o valor que deverá ser inserido na base de dados
            field_name, field_value = self.validate.field_update(item_scheme_name, item_scheme, field_value, "custom_update")

            # Verifica se o campo não está com erro de validação
            if field_value != '__error__' and field_value != '__ERROR__':
                # Registra o campo na lista de dados que deverá ser enviada para o servidor de base de dados
                if field_name != "id":
                    list_update_data.append({"field": field_name, "value": field_value})

        # Prepara a lista de campos que será enviada para o servidor de base de dados
        # Prepara a lista de campos que devrá ser retornada do servidor depois de salvar na base de dados
        list_update_data = self.prepare_update_data(list_update_data)

        # Verificar se tem algo para gravar no banco, caso contrário, registra mensagem de error
        if len(list_update_data) <= 0:
            self.status.error("NO_DATA_TO_UPDATE", None, [])

        # Retorna a lista de dados que deverá ser enviada para o servidor de base de dados
        return list_update_data

    # --------------------------------
    # __insert_data_join
    # --------------------------------
    # Função recursiva utilizada para tratar conflito de elementos filho no momento de gravar na base de dados
    #
    # data - Objeto contendo dados existentes do mesmo grupo ou um objeto vazio para receber novos elementos
    # value - Dados que deverão ser inseridos no objeto
    def __insert_data_join(self, data, value):
        # Pega cada elemento do objeto valor
        for item in value:
            # Verifica se valor é um objeto
            # Verifica se o elemento filho valor é um objeto
            # Verifica se o elemento filho existe no objeto de dados
            if type(value) == dict and type(value[item]) == dict and type(data) == dict and item in data.keys():
                # Adiciona novo elemento no objeto de dados com o resultado da interação nos objetos filhos do objeto valor
                data[item] = self.__insert_data_join(data[item], value[item])

            # Verifica se valor é um objeto
            # Verifica se o elemento filho valor é um objeto
            elif type(value) == dict and type(data) == dict:
                # Adiciona o objeto valor como novo elemento no objeto de dados
                data[item] = value[item]

            # Verifica se o objeto dados é um objeto
            elif type(data) == dict:
                # Adiciona o valor como novo elemento no objeto de dados
                data[item] = value

        # Retorna os dados inseridos
        return data

    # --------------------------------
    # __prepare_insert_data_fields
    # --------------------------------
    # Prepara lista de campos que podem ser incluidos no documento
    # Prepara e registra a lista de campos que deverá ser retornada do servidor após a inclusão do documento na base de dados
    def __prepare_insert_data_fields(self, list_data):
        result = dict()
        fields = dict()

        for item in list_data:
            # Verifica se o nome do campo existe na lista que será incluida no documento
            if (item["field"]) in result.keys():
                # Caso não esteja na lista então o campo é incluído
                # Prepara objetos-filhos no padrão permitido para registrar na base de dados
                result[(item["field"])] = self.__insert_data_join(result[(item["field"])], item["value"])
            else:
                # Registra o campo e valor na lista de campos válidos do documento
                result[(item["field"])] = item["value"]

            # Adiciona o campo na lista de campos que devem ser retornados do servidor
            fields[(item["field"])] = True

        # Registra campos que devem ser retornados
        self.datarest.fields(fields)

        # Retorna dicionário de dados do documento
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#import i18n
import yaml
from actions import Actions
from devdoo_service import DevdooService
from devdoo_database import DevdooDatabase
from broker import Broker
from brokers import Brokers
from check import Check
from config import Config
from connection import Connection
from convert import Convert
from devdoo_broker import DevdooBroker
from data_response import DataResponse
from database import Database
from datafilter import DataFilter
from datarest import DataRest
from endpoint import Endpoint
from yml import Yml
from pack import Pack
from response import Response
from stack import Stack
from stacks import Stacks

from services import Services
from service_db import ServiceDB
from socket import Socket
from status import Status
from validate import Validate




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pprint import pprint
from check import Check

class Endpoint:

    def __init__(self, uri, endpoint, status):
        # pprint(endpoint)
        self.__ready = False
        self.status = status
        self.uri = uri

        self.__decode(endpoint)

    # --------------------------------
    # __decode
    # --------------------------------
    def __decode(self, endpoint):
        self.action = endpoint["action"]
        self.collection = endpoint["collection"]
        self.scheme = endpoint["scheme"]
        self.type = endpoint["type"]
        self.max_limit = endpoint["max_limit"]
        self.__fields = endpoint["fields"]
        self.__check_params()

    def fields(self, is_public_access = True):
        if is_public_access is True:
            return self.__fields["public"]
        else:
            return self.__fields["default"]


    def ready(self):
        return self.__ready

    # --------------------------------
    # show
    # --------------------------------
    # Imprime propriedade/valor do objeto
    def show(self, message=None):
        if message:
            print message, ":::------->>>"
        pprint(self.__dict__)

    def is_field_valid(self, field, status):
        # Caso o campo nao esteja registrado no scheme, registra na lista de campos com erro
        if field not in self.fields(False):
            print "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXSSSSSSSSSSSSS", field
            # Caso o campo nao esteja registrado no scheme, registra mensagem de error
            status.error("DOES_NOT_EXIST_FIELD", None, [field])

    # --------------------------------
    # __check_params
    # --------------------------------
    # Verifica se o pacote de comunicação é válido
    def __check_params(self):
        errors = []

        # Verifica se foi definido a ação que será executado no base de dados
        if Check.is_empty(self.action) is True:
            errors.append("ACTION")

        # Verifica se foi definido a coleção que deverá ser usada
        if Check.is_empty(self.collection) is True:
            errors.append("COLLECTION")

        # Verifica se foi definido a lista de campos da coleção
        if Check.is_object(self.__fields) is not True:
            errors.append("FIELDS")

        # Verifica se foi definido schema da coleção
        if Check.is_object(self.scheme) is not True:
            errors.append("SCHEME")

        # Verifica se foi definido o tipo de retorna de dados do base de dados (item|list)
        if Check.is_empty(self.type) is True:
            errors.append("TYPE")

        # Verifica se foi definido o limit máximo de dados retornados do base de dados
        if type(self.max_limit) is not int:
            errors.append("MAX_LIMIT")

        if len(errors) > 0:
            self.status.error("INVALID_ENDPOINT_PARAMS", None, [(",".join(errors)), self.uri])
        else:
            self.__ready = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from connection import Connection
from check import Check
from convert import Convert
from endpoint import Endpoint
from pprint import pprint
from brokers import Brokers
import time

class Database:

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, database_id, status, service_port=None):
        self.status = status
        self.connection = Connection(self.status)
        self.__endpoints = dict()
        self.collections = None

        # Identificador do broker
        self.id = database_id
        # Configurações da Rede utilizada pelo broker
        self.network = None
        self.actions = None
        self.method = None
        self.__config(service_port)

    # --------------------------------
    # __setitem__
    # --------------------------------
    def __setitem__(self, field, value):
        self.field = value

    # --------------------------------
    # endpoint
    # --------------------------------
    def endpoint(self, method, source, status):
        print "[[[[[[[ENDPOINT]]]]]]]"
        print 'METHOD:', method
        print 'SOURCE:', source
        print 'STATUS:', status
        print '========================================='
        print 'SELF.ACTIONS:'
        pprint(self.actions['put']['/1.0/gdi/products/mercado/editar'])
        #time.sleep(5)
        print '========================================='
        pprint(self.actions)
        self.method = method
        return Endpoint(source, self.actions[method][source], status)

    # --------------------------------
    # endpoint
    # --------------------------------
    #def endpoint(self, source):
    #    if source in self.__endpoints.keys():
    #        return self.__endpoints[source]
    #    else:
    #        return None

    # --------------------------------
    # show
    # --------------------------------
    def ready(self):
        return self.status.ready()

    # --------------------------------
    # show
    # --------------------------------
    # Imprime propriedade/valor do objeto
    def show(self, message=None):
        if message:
            print message, ":::------->>>"
        pprint(self.__dict__)

    # --------------------------------
    # __config
    # --------------------------------
    # Implementa auto configuração do broker
    # Vai até o servidor de configuração buscar informações de configuração para o broker
    def __config(self, service_port):
        if self.id:
            data_response = self.connection.load_config(self.id)

            if data_response.ready():
                data = data_response.values()

                for field in data:
                    self.__dict__[field] = data[field]
                self.__output()
                self.__prepare_scheme()
            else:
                self.status.error("ERROR_DATABASE_CONFIG", None,
                                  ["Nao obteve dados de configuracao do database", self.id])
        else:
            self.status.error("ERROR_DATABASE_CONFIG", None, ["Nao recebeu identificador de database", "--"])

        if not self.status.ready():
            self.status.to_print()

    # --------------------------------
    # __brokers
    # --------------------------------
    def __brokers(self, service_port):
        if self.brokers:
            self.brokers = Brokers(service_port, self.brokers, self.status, False, True)
        else:
            self.status.error("ERROR_DATABASE_CONFIG", None,
                              ["Nao obteve dados de configuracao de brokers do database", self.id])

    # --------------------------------
    # __output
    # --------------------------------
    def __output(self):
        if self.network:
            self.__brokers(self.network["port"])
        else:
            self.status.error("ERROR_DATABASE_CONFIG", None,
                              ["Nao obteve dados de configuracao do network no database", self.id])


    # --------------------------------
    # fields_insert
    # --------------------------------
    # Prepara a lista de campos que pode ser incluída na base de dados
    #
    def fields_insert(self, body, endpoint):
        list_scheme_clean = dict()
        # Verifica se o campo tem valor default
        for item_name in endpoint.scheme:
            # Pega um esquema da lista
            item_scheme = endpoint.scheme[item_name]
            # Verifica se o campo tem valor default ou é requerido
            if Check.is_default(item_scheme) or self.__required(item_scheme):
                # Registra o dot field name do campo
                item_scheme["field"] = item_name
                # Verifica se o valor do campo não é vazio
                # Registra o valor do campo
                item_scheme["value"] = Convert.to_empty(item_scheme, self.__default(item_scheme))
                # Registra o schema válido na lista de esquema limpa
                list_scheme_clean[item_name] = item_scheme

        # Processa todos os campos de body para colocar em letra minuscula e criar lista de campos validos
        for item_body_field in body:
            # Processa o nome do campo recebido no body
            body_field = item_body_field.replace(' ', '_').lower().strip()

            # Verifica se o campo recebido via body esta configurado no esquema
            item_name, item_scheme = self.__check_body_item_scheme_insert(body_field, endpoint.scheme)

            # Caso o campo recebido via body está registrado no esquema então adiciona na lista de esquema limpa
            if item_scheme is not None:
                # Registra o dot field name do campo
                item_scheme["field"] = item_name
                # Verifica se o valor do campo não é vazio
                # Registra o valor do campo
                item_scheme["value"] = Convert.to_empty(item_scheme, body[item_body_field])
                # Registra o schema válido na lista de esquema limpa
                list_scheme_clean[item_name] = item_scheme

            # Verifica se o campo é válido
            endpoint.is_field_valid(item_body_field, self.status)

        return list_scheme_clean

    # --------------------------------
    # fields_update
    # --------------------------------
    def fields_update(self, body, scheme, fields_default):
        list_scheme_clean = dict()

        # Processa todos os campos de body para colocar em letra minuscula e criar lista de campos validos
        for item_body_field in body:
            body_field = item_body_field.replace(' ', '_').lower().strip()
            item_name, item_scheme = self.__check_body_item_scheme_update(body_field, scheme)

            #print 'FIELDS_UPDATE', item_name, item_scheme
            #print 'BODY_FIELD>>>>>>>>>', body_field
            #print 'SCHEME>>>>>>>>>>>>>', scheme

            if item_scheme is not None:
                item_scheme["field"] = item_name
                item_scheme["value"] = Convert.to_empty(item_scheme, body[item_body_field])
                list_scheme_clean[item_name] = item_scheme

            # Caso o campo nao esteja registrado no scheme, registra mensagem de error
            if item_body_field not in fields_default:
                self.status.error("DOES_NOT_EXIST_FIELD", None, [item_body_field])

        return list_scheme_clean

    # --------------------------------
    # __check_body_item_scheme_insert
    # --------------------------------
    # Verifica se o campo recebido via body esta configurado no esquema
    @staticmethod
    def __check_body_item_scheme_insert(body_field, scheme):
        for item_scheme in scheme:
            if item_scheme == body_field:
                return body_field, scheme[item_scheme]
        return None, None

    # --------------------------------
    # __check_body_item_scheme_update
    # --------------------------------
    # Verifica se o campo recebido via body esta configurado no esquema
    @staticmethod
    def __check_body_item_scheme_update(body_field, scheme):
        for item_scheme in scheme:
            if item_scheme['field'] == body_field:
                return body_field, item_scheme
        return None, None

    # --------------------------------
    # __default
    # --------------------------------
    @staticmethod
    def __default(item_scheme):
        value = ""
        if 'default' in item_scheme.keys():
            value = item_scheme['default']
        return value

    # --------------------------------
    # __required
    # --------------------------------
    @staticmethod
    def __required(item_scheme):
        value = False
        if 'required' in item_scheme.keys():
            value = item_scheme['required']
        return value

    # --------------------------------
    # __collection
    # --------------------------------
    def __collection(self, uri, item, collections):
        collection = None
        name = None
        if "collection" in item.keys():
            name = item["collection"]
            if name in collections.keys():
                collection = collections[name]["schemes"]
            else:
                self.status.error("COLLECTION_INVALID", None, [name])
        else:
            self.status.error("COLLECTION_INVALID_IDENTIFIER", None, [uri])

        return name, collection

    # --------------------------------
    # __fields_public
    # --------------------------------
    @staticmethod
    def __fields_public(method, item, schemes):
        for i in schemes:
            if "public" in schemes[i].keys():
                if method != "put":
                    item["scheme"][i] = schemes[i]
                if schemes[i]["public"] is True:
                    (item["fields"]["public"]).append(schemes[i]["field"])

        return item

    # --------------------------------
    # __put_scheme
    # --------------------------------
    def __put_scheme(self, put, schemes):
        for item in put:
            index = 0
            for field in put[item]:
                put[item][index] = schemes[(put[item][index])]
                index = index + 1
        return put

    # --------------------------------
    # __prepare_scheme
    # --------------------------------
    def __prepare_scheme(self):
        collections = self.collections
        actions = self.actions

        for method in actions:
            endpoint = actions[method]
            for item in endpoint:
                name, schemes = self.__collection(item, endpoint[item], collections)
                if schemes is not None:
                    if self.__check_endpoint(item, endpoint[item]):
                        endpoint[item] = self.__fields_public(method, endpoint[item], schemes)
                        endpoint[item]["fields"]["default"] = self.__check_conflit_fields(endpoint[item]["fields"]["default"])
                        endpoint[item]["fields"]["public"] = self.__check_conflit_fields(endpoint[item]["fields"]["public"])
                        if method == "put":
                            endpoint[item]["scheme"] = self.__put_scheme(endpoint[item]["scheme"], schemes)

                        self.__endpoints[item] = Endpoint(item, endpoint[item], self.status)

                        #if method == "put":
                        #    pprint(endpoint[item])

    # --------------------------------
    # __check_endpoint
    # --------------------------------
    def __check_endpoint(self, uri, endpoint):
        errors = []
        if "action" not in endpoint.keys():
            errors.append("ACTION")

        if "collection" not in endpoint.keys():
            errors.append("COLLECTION")

        if "fields" not in endpoint.keys():
            errors.append("FIELDS")

        if "scheme" not in endpoint.keys():
            errors.append("SCHEME")

        if "type" not in endpoint.keys():
            errors.append("TYPE")

        if "max_limit" not in endpoint.keys():
            errors.append("MAX_LIMIT")

        if len(errors) > 0:
            self.status.error("INVALID_ENDPOINT_PARAMS", None, [(",".join(errors)), uri])
            return False
        else:
            return True

    # --------------------------------
    # __check_conflit_fields
    # --------------------------------
    # Verifica se existe conflito de nomes de campos em modo desenvolvimento de microserviços
    # Não permite incluir na lista campos-pai que possuem campos-filhos que podem sobrepor informações
    def __check_conflit_fields(self, fields_default):
        fields_clean = []
        fields_errors = dict()

        # Processa a lista de campos default
        for item_check in fields_default:
            # Retira um item da lista para comparar com outros itens da mesma lista
            for item_field in fields_default:
                # Caso um campo esteja em conflito com outros campos é adicionado na lista de erros
                if item_check + '.' in item_field:
                    fields_errors[item_check] = True

        # Gera a lista de campos default sem os campos que foram encontrados conflitos
        for item_field in fields_default:
            if item_field not in fields_errors.keys():
                fields_clean.append(item_field)

        if len(fields_errors) > 0:
            self.status.error("FIELD_CONFLICT", None, [(",".join(fields_errors))])

        return fields_clean



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from check import Check
from pprint import pprint

#TODO:: Remover classe Config

class Config(object):

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, params, status):
        self.__ready = False
        self.id = None
        self.database = None
        self.brokers = None
        self.endpoint = None
        self.name = None
        self.type = None
        self.services = None
        self.status = status
        self.__init(params)

    # --------------------------------
    # __init
    # --------------------------------
    def __init(self, params):

        print "PARAMS", params.network

        if (params is not None) and params.ready():
            # Identificador do serviço
            self.id = Check.value_string("id", params.id, self.status)
            # Identificador do tipo de serviço (devdoo-broker|devdoo-database|devdoo-service)
            self.type = Check.value_string("type", params.type, self.status)

            if self.type == "devdoo-broker":
                self.services = Check.value_list("services", params.services, self.status)
                if Check.is_object(params.network):
                    self.network = params.network
                    self.endpoint = "tcp://*:" + str(self.network["port"])

            elif (self.type == "devdoo-service"):
                # Nome do serviço
                self.name = Check.value_string("name", params.name, self.status)
                #self.mongodb = Check.value_dict("mongodb", params.mongodb, self.status)
                #self.endpoint = Check.value_dict("endpoint", params.database, self.status)

            elif (self.type == "devdoo-database"):
                # Nome do serviço
                self.mongodb = Check.value_dict("mongodb", params.mongodb, self.status)
                self.collections = Check.value_dict("collections", params.mongodb, self.status)
                self.actions = Check.value_dict("actions", params.mongodb, self.status)
        else:
            self.status.error("INVALID_CONFIG", None, ["Falha na leitura de arquivo de configuracao"])

        return self.status.ready()

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        if self.status.ready() is False:
            pprint(self.status.to_list())
        return self.status.ready()

    def show(self):
        pprint(self.__dict__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from connection import Connection
from brokers import Brokers
from database import Database
from pprint import pprint

# --------------------------------
# Service
# --------------------------------
'''
A classe Service gerencia todas as interações do serviço
'''
class Service:

    # --------------------------------
    # __init__
    # --------------------------------
    '''
    Inicializa a classe

    :param service_id: Identificador do serviço
    :param status: Gerenciador de erros
    :param has_connection_backend: Indica se deve configurar endpoind para conexão backend
    '''
    def __init__(self, service_id, status, has_connection_backend=False):
        print ('SERVICE_ID =======>', service_id)
        # Variaveis default
        self.brokers = None
        self.database = None
        self.network = None

        # Variaveis de parametros
        self.id = service_id
        self.status = status
        self.has_connection_backend = has_connection_backend

        # Variaveis inicializadas
        self.connection = Connection(self.status)

        # Verifica se o arquivo de configuração foi inicializado corretamente
        if self.__has_config() is not True:
            self.status.error("INVALID_SERVICE", None, ["FALHA NA INICIALIZACAO DO SERVICO", service_id])
            self.status.to_print()
        else:
            self.status.show("SERVICE_INIT", [self.id])

    # --------------------------------
    # __setitem__
    # --------------------------------
    '''
    Configura a classe para criar propriedades dinamicamente 
    '''
    def __setitem__(self, field, value):
        self.field = value

    # --------------------------------
    # broker_connect_backend
    # --------------------------------
    '''
    Recupera o endpoint de conexão connect_backend
    '''
    def broker_connect_backend(self):
        if self.brokers is not None:
            broker = self.brokers.available()
            return broker.connect_backend
        else:
            self.status.error("INVALID_BROKER", None, ["Nao foi encontrado broker disponivel para obter o 'connect_backend'.", self.id])

    # --------------------------------
    # database_connect_frontend
    # --------------------------------
    '''
    Recupera o endpoint de conexão connect_frontend
    '''
    def database_connect_frontend(self):
        if (self.database is not None) and (self.database.brokers is not None):
            database = self.database.brokers.available()
            return database.connect_frontend
        else:
            self.status.error("INVALID_BROKER", None, ["Nao foi encontrado database broker disponivel para obter o 'connect_frontend'.", self.id])

    # --------------------------------
    # endpoint_backend
    # --------------------------------
    '''
    Obtém o endereço utilizado pelo serviço para conectar o broker seguinte
    '''
    def endpoint_backend(self):
        broker = self.brokers.available()
        # Verifica se o objeto de broker foi criado e se está válido
        if broker and broker.ready():
            return broker.bind_backend
        else:
            self.status.error("INVALID_SERVICE", None, ["Nao foi encontrado broker disponivel par obter o 'endpoint_backend'.", self.id])

    # --------------------------------
    # ready
    # --------------------------------
    '''
    Status indica se está pronto
    
    :return: void
    '''
    def ready(self):
        return self.status.ready()

    # --------------------------------
    # show
    # --------------------------------
    '''
    Imprime propriedade/valor do objeto

    :param title: Título identificador
    :return: void
    '''
    def show(self, title=None):
        if title is not None:
            print title, ":::------->>>"
        pprint(self.__dict__)

    # --------------------------------
    # __has_config
    # --------------------------------
    '''
    Implementa auto configuração do serviço
    Vai até o servidor de configuração buscar informações
    
    :return: boolean
    '''
    def __has_config(self):
        if self.id:
            print "SERVICE.__HAS_CONFIG"
            # Recupera configurações no servidor de serviços API
            data_response = self.connection.load_config(self.id)

            # Verifica se a resposta do servidor é válida
            if data_response.ready():
                # Registra na classe as propriedades da configuração
                data = data_response.values()
                for field in data:
                    self.__dict__[field] = data[field]


                # Configura serviços-filhos
                self.__config_database()
                self.__config_network()
        else:
            self.status.error("ERROR_SERVICE_CONFIG", None, ["Não recebeu identificador de serviço"])

        return self.status.ready()

    # --------------------------------
    # __config_brokers
    # --------------------------------
    '''
    Implementa configurações de brokers-filhos do serviço

    :return: void
    '''
    def __config_brokers(self, service_port):
        if self.brokers and self.network:
            self.brokers = Brokers(service_port, self.brokers, self.status, self.has_connection_backend)
        else:
            self.status.error("ERROR_SERVICE_CONFIG", None, ["Nao obteve dados de configuracao de brokers do servico", self.id])

    # --------------------------------
    # __config_database
    # --------------------------------
    '''
    Implementa configurações de database-filho do serviço

    :return: void
    '''
    def __config_database(self):
        print "SELF.DATABASE", self.database
        if self.database is not None:
            self.database = Database(self.database, self.status, self.network["port"])

    # --------------------------------
    # __network
    # --------------------------------
    '''
    Implementa configuração de rede do serviço

    :return: void
    '''
    def __config_network(self):
        if self.network:
            self.__config_brokers(self.network["port"])
        else:
            self.status.error("ERROR_BROKER_CONFIG", None, ["Nao obteve dados de configuracao do network no service", self.id])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import yaml
import sys
import os
from check import Check
from pprint import pprint

class Yml:

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, status):
        self.status = status

    # --------------------------------
    # service_id
    # --------------------------------
    # Obtém o identificador do serviço
    def id(self):
        # Tenta pegar configurações docker
        id = os.environ.get('DEVDOO')

        # Caso não consiga pegar o id do serviço no docker
        if not id:
            # Tenta pegar configurações de arquivo local
            id = self.__config()
        if not id:
            self.status.error("CONFIGURATION_SERVICE", None,
                              ["Nao recebeu identificador de arquivo de configuracao (service_id)"])
        return id

    # --------------------------------
    # __config
    # --------------------------------
    # Recupera informações de configuração em arquivo yml local
    def __config(self):
        file_open = None

        if len(sys.argv) > 1:
            # Recupera o endereço do arquivo de configuração
            path_file = str(sys.argv[1])
            # Verifica se o endereço do arquivo é válido
            if (type(path_file) == str) and (".yml" in path_file):
                try:
                    # Abre o arquivo
                    file_yml = open(path_file, "r")
                    # Converte o arquivo em objeto
                    file_open = yaml.load(file_yml)
                except Exception:
                    self.status.error("CONFIGURATION_SERVICE", None, ["Arquivo de configuracao YML nao encontrado"])
                    pass
            else:
                self.status.error("CONFIGURATION_SERVICE", None,
                                  ["Identificador do servico NAO fornecido na inicializacao"])

            # Verifica se o arquivo é válido e se existe configurações de serviços nele
            if (file_open is not None) and ("services" in file_open.keys()):
                # Processa o arquivo
                for item in file_open["services"]:
                    services = file_open["services"][item]
                    if "environment" in services:
                        environment = services["environment"]
                        if "DEVDOO" in environment:
                            # Recupera as configurações devdoo
                            value = environment["DEVDOO"]
                            # Verifica se recebeu configuração de devdoo válida
                            if (Check.is_string(value) is True) and (Check.is_empty(value) is False):
                                return value
                            else:
                                self.status.error("CONFIGURATION_SERVICE", None,
                                                  ["Arquivo de configuracao nao e um 'DEVDOO' valido"])
                        else:
                            self.status.error("CONFIGURATION_SERVICE", None,
                                              ["Arquivo de configuracao encontrou falhas no grupo 'DEVDOO'"])
                    else:
                        self.status.error("CONFIGURATION_SERVICE", None,
                                          ["Arquivo de configuracao encontrou falhas no grupo 'environment'"])
            else:
                self.status.error("CONFIGURATION_SERVICE", None,
                                  ["Arquivo de configuracao nao possui o grupo 'services'"])
        else:
            self.status.error("CONFIGURATION_SERVICE", None,
                              ["Nao foi indicado nenhum endereco de arquivo de configuracao"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# TODO:: Refatorar completamente a classe Status
# TODO:: Implementar multi-idiomas classe Status
# TODO:: Remover de dentro da classe Status todas as mensagens
class Status:

    def __init__(self):
        # Flags de erros/alertas
        self.__has_error = False
        self.__has_info = False
        self.__has_log = False
        self.__has_warn = False

        # Lista de erros
        self.__list_errors = []

        # Mensagens de inicialização dos serviços
        self.show_messages = {
            "MONGO_CLIENT": {
                "message": "CRIADO::-->Banco de dados MongoDB"
            },
            "SERVICE_ID": {
                "message": "SERVICE-ID::{0}"
            },
            "REGISTER_SERVICE": {
                "message": "REGISTER-SERVICE-ID::{0}"
            },
            "BROKER_INIT": {
                "message": "INIT-BROKER::{0}"
            },
            "SERVICE_INIT": {
                "message": "INIT-SERVICE::{0}"
            },
            "DEVDOO_SERVICE_INIT": {
                "message": "DEVDOO-SERVICE-INIT::{0}, {1}, {2}"
            },
            "DEVDOO_BROKER_INIT": {
                "message": "INIT-DEVDOO-BROKER::{0}"
            },
            "CONFIG_INIT": {
                "message": "INIT-CONFIG::{0}"
            }
        }

        # Mensagens de erro
        self.error_messages = {
            "TYPE_INVALID_ARRAY": {
                "code": 4505845,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize um valor do tipo array [\"a\", \"b\", 1, 2]",
                "message_except": "ERRO ARRAY no campo {0}, {1}"
            },
            "TYPE_INVALID_BOOLEAN": {
                "code": 5454575,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize um valor entre true | false | 0 | 1",
                "message_except": "ERRO BOOLEAN no campo {0}, {1}"
            },
            "TYPE_INVALID_DATE": {
                "code": 5454587,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize os formatos de data AAAA-MM-DD ou AAAA-MM-DDTHH:MM:SS.mmm",
                "message_except": "ERRO DATE no campo {0}, {1}"
            },
            "TYPE_INVALID_DECIMAL": {
                "code": 8545255,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize um valor entre -x ate x.",
                "message_except": "ERRO DECIMAL no campo {0}, {1}"
            },
            "TYPE_INVALID_ENUM": {
                "code": 4505845,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize uma das opcões de valor : {2}",
                "message_except": "ERRO ENUM no campo {0}, {1}"
            },
            "TYPE_INVALID_NUMBER": {
                "code": 8545255,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize um valor entre -9223372036854775808 ate 9223372036854775807.",
                "message_except": "ERRO NUMBER no campo {0}, {1}"
            },
            "TYPE_INVALID_OBJECT": {
                "code": 545465454,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize um valor do tipo object json.",
                "message_except": "ERRO OBJECT no campo {0}, {1}"
            },
            "TYPE_INVALID_OBJECT_ID": {
                "code": 6565482,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize um valor do tipo object_id, exemplo: '5abcd65b8356694714f893e9'.",
                "message_except": "ERRO OBJECT_ID no campo {0}, {1}"
            },
            "TYPE_INVALID_TIMESTAMP": {
                "code": 32456454,
                "message": "O valor do campo '{0}' nao pode ser '{1}', vazio ou null, utilize um valor do tipo timestamp, exemplo: '1522343386'.",
                "message_except": "ERRO TIMESTAMP no campo {0}, {1}"
            },
            "DOES_NOT_EXIST_FIELD": {
                "code": 34563456,
                "message": "O campo '{0}' nao existe no microservico.",
                "message_except": "ERRO FIELD campo {0}, {1}"
            },
            "DOES_NOT_EXIST_FIELD_TYPE": {
                "code": 5454542,
                "message": "No campo '{0}' o tipo configurado '{1}', nao existe.",
                "message_except": "ERRO FIELD no campo {0}, {1}"
            },
            "MAX_LENGTH": {
                "code": 65734747,
                "message": "No campo '{0}' O tamanho maximo permitido e '{1}', foi recebido '{2}.",
                "message_except": "ERRO MAX_LENGTH no campo {0}, {1}"
            },
            "MIN_LENGTH": {
                "code": 4567567,
                "message": "No campo '{0}' O tamanho minimo permitido e '{1}', foi recebido '{2}.",
                "message_except": "ERRO MIN_LENGTH no campo {0}, {1}"
            },
            "MIN_VALUE": {
                "code": 4564568,
                "message": "No campo '{0}' O valor minimo permitido e '{1}', foi recebido '{2}.",
                "message_except": "ERRO MIN_VALUE no campo {0}, {1}"
            },
            "MAX_VALUE": {
                "code": 7685685,
                "message": "No campo '{0}' O valor maximo permitido e '{1}', foi recebido '{2}.",
                "message_except": "ERRO MAX_VALUE no campo {0}, {1}"
            },
            "FIELD_REQUIRED": {
                "code": 8484545,
                "message": "O campo '{0}' e requerido.",
                "message_except": "ERRO FIELD_REQUIRED no campo {0}, {1}"
            },
            "SERVICE_FAILURE": {
                "code": 6545454,
                "message": "Identificador do servico nao valido:: '{0}' != '{1}' :: '{2}' != '{3}'.",
                "message_except": "ERRO FALHA DE SERVICE {0}, {1}"
            },
            "MAX_LIMIT_EXCEEDED": {
                "code": 85545454,
                "message": "O limite maximo ({0}) foi excedido. Limite solicitado ({1}).",
                "message_except": "ERRO MAX LIMIT EXCEEDED {0}, {1}"
            },
            "INVALID_ENDPOINT": {
                "code": 564564654,
                "message": "Endpoint nao registrado '{0}' no microservico '{1}' para o metodo '{2}'.",
                "message_except": "ERRO INVALID_ENDPOINT {0}, {1}"
            },
            "INVALID_ACTION": {
                "code": 5646546517,
                "message": "Acao '{0}' nao registrada no microservico '{1}' - '{2}' - para o metodo '{3}'.",
                "message_except": "ERRO INVALID ACTION {0}, {1}"
            },
            "INVALID_ACTION_DB": {
                "code": 654654654,
                "message": "Acao '{0}' nao disponivel para interacões base de dados.",
                "message_except": "ERRO INVALID ACTION DB {0}, {1}"
            },
            "FIELD_CONFLICT": {
                "code": 5646546517,
                "message": "O campo '{0}' nao pode ser utilizado por existir campos filhos.",
                "message_except": "ERRO FIELD CONFLICT {0}, {1}"
            },
            "NO_DATA_TO_INSERT": {
                "code": 545645645,
                "message": "Nao foi enviado dados para ser inserido na base de dados. {0}",
                "message_except": "ERRO NO DATA TO INSERT {0}, {1}"
            },
            "NO_DATA_TO_UPDATE": {
                "code": 34545645632,
                "message": "Nao foi enviado dados para ser atualizado na base de dados.",
                "message_except": "ERRO NO DATA TO UPDATE {0}, {1}"
            },
            "INVALID_SCHEME_UPDATE": {
                "code": 645456653,
                "message": "Esquema de update Invalido nao encontrado $set, $inc...",
                "message_except": "ERRO INVALID SCHEME UPDATE {0}, {1}"
            },
            "WARN_FIND_ITEM": {
                "code": 5356456445,
                "message": "O filtro da busca encontrou mais mais itens do que o esperado, nenhum item e retornado.",
                "message_except": "WARN FIND ITEM {0}, {1}"
            },
            "WARN_INSERT_ITEM": {
                "code": 5356456445,
                "message": "Muitos documentos foram inseridos utilizando um endpoit do tipo 'item', somente o primeiro documento e retornado.",
                "message_except": "WARN INSERT ITEM {0}, {1}"
            },
            "SERVICE_INVALID": {
                "code": 66565,
                "message": "Microservice parametro invalido :: {1} em {0}.",
            },
            "MICROSERVICE_INVALID": {
                "code": 36645764,
                "message": "Microservice invalido :: Service ID:: {1}.",
            },
            "CONFIGURATION_SERVICE": {
                "code": 24545454,
                "message": "{0}"
            },
            "INVALID_PACKAGE": {
                "code": 84545754,
                "message": "{0}"
            },
            "INVALID_PACKAGE_PARAMS": {
                "code": 545454545,
                "message": "{0} :: {1}"
            },
            "INVALID_SERVICE": {
                "code": 54621854,
                "message": "{0}--{1}"
            },
            "INVALID_BROKER": {
                "code": 684484,
                "message": "{0}--{1}"
            },
            "FAIL_LOAD_FILE_CONFIG": {
                "code": 855618,
                "message": "{0}--{1}"
            },
            "COLLECTION_INVALID": {
                "code": 654545,
                "message": "Nome de colecao registrada no servico invalido em '{0}'."
            },
            "COLLECTION_INVALID_IDENTIFIER": {
                "code": 65465658,
                "message": "Identificador da colecao invalido ou nao definido em '{0}'."
            },
            "INVALID_ENDPOINT_PARAMS": {
                "code": 84545454,
                "message": "Parametros de configuracao do de enpoint nao definido ou invalido: '{0}' na uri '{1}'."
            },
            "INVALID_DATABASE_CONFIG": {
                "code": 845487545,
                "message": "Configuracao do base de dados falhou: '{0}'."
            },
            "INVALID_NETWORK_CONFIG": {
                "code": 845485748,
                "message": "Configuracao de rede falhou: '{0}'."
            },
            "INVALID_CONFIG": {
                "code": 8754557,
                "message": "Nao recebeu parametro de configuracao: '{0}'."
            },
            "ERROR_BROKER_CONFIG": {
                "code": 854520854,
                "message": "ERROR_BROKER_CONFIG: '{0}', '{1}'."
            },
            "ERROR_SERVICE_CONFIG": {
                "code": 887854,
                "message": "ERROR_SERVICE_CONFIG: '{0}', '{1}'."
            },
            "ERROR_DATABASE_CONFIG": {
                "code": 4542454,
                "message": "ERROR_DATABASE_CONFIG: '{0}', '{1}'."
            },
            "RESPONSE": {
                "code": 52454245454,
                "message": "ERROR_RESPONSE: '{0}', '{1}'."
            },
            "DATA_RESPONSE": {
                "code": 85456455,
                "message": "ERROR_DATA_RESPONSE: '{0}'."
            },
            "ERROR_STACK": {
                "code": 865454952,
                "message": "ERROR_STACK: '{0}'."
            }

        }

    # --------------------------------
    # addx
    # --------------------------------
    def addx(self, code, value, params):
        error = self.error_messages[code]
        self.add(error["code"], error["message"].format(*params))

        if (value is not None) and (type(value) == dict) and ("error" in value.keys()):
            params = [(params[1]), (value["error"])]
            self.add(error["code"], error["message_except"].format(*params))

    # --------------------------------
    # alerts
    # --------------------------------
    def alerts(self, code, value, params):
        self.__has_error = True
        self.addx(code, value, params)

    # --------------------------------
    # error
    # --------------------------------
    def error(self, code, value, params):
        self.__has_error = True
        self.addx(code, value, params)

    # --------------------------------
    # show
    # --------------------------------
    def show(self, code, params=None):
        if params is not None:
            print self.show_messages[code]["message"].format(*params)
        else:
            print self.show_messages[code]["message"]

    # --------------------------------
    # info
    # --------------------------------
    def info(self, code, value, params):
        self.__has_info = True
        self.addx(code, value, params)

    # --------------------------------
    # log
    # --------------------------------
    def log(self, code, value, params):
        self.__has_log = True
        self.addx(code, value, params)

    # --------------------------------
    # warn
    # --------------------------------
    def warn(self, code, value, params):
        self.__has_warn = True
        self.addx(code, value, params)

    # --------------------------------
    # add_errors
    # --------------------------------
    # Adiciona uma lista de erros na lista de erros retornada
    #
    def add_errors(self, list_errors):
        if list_errors is not None:
            for error in list_errors:
                self.add(error['code'], error['message'])

    # --------------------------------
    # add
    # --------------------------------
    def add(self, code, message):
        self.__has_error = True
        self.__list_errors.append(
            {"code": code, "message": message}
        )

    # --------------------------------
    # add_required
    # --------------------------------
    def add_required(self, code, message):
        self.__has_error = True
        self.__list_errors.append(
            {"code": code, "message": message}
        )

    # --------------------------------
    # add_field_required
    # --------------------------------
    def add_field_required(self, field, code, message):
        self.__has_error = True
        self.__list_errors.append(
            {"field": field, "code": code, "message": message}
        )

    # --------------------------------
    # to_list
    # --------------------------------
    def to_list(self):
        if len(self.__list_errors) > 0:
            return self.__list_errors
        return None

    # --------------------------------
    # to_print
    # --------------------------------
    def to_print(self):
        if len(self.__list_errors) > 0:
            print self.__list_errors
        return None

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__has_error is not True

    # --------------------------------
    # has_error
    # --------------------------------
    def has_error(self):
        return self.__has_error



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq

# --------------------------------
# Socket
# --------------------------------
class Socket:

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, service):
        self.__ready = False
        self.__connect_backend = None
        self.backend = None
        self.__service = service

        # Verifica se o serviço existe e está pronto
        if (service is not None) and service.ready():
            self.backend = service.endpoint_backend()

            # Cria contexto ZMQ
            context = zmq.Context()
            
            # Abre portas para os Serviços no backend
            self.__connect_backend = context.socket(zmq.DEALER)
            self.__connect_backend.bind(service.endpoint_backend())
            self.__ready = True

    # --------------------------------
    # connect_backend
    # --------------------------------
    def connect_backend(self):
        return self.__connect_backend

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.__ready



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
import uuid
from pack import Pack
from status import Status

class Stack:
    """Um pedido que entrou no sistema."""

    def __init__(self, message, timeout, tries):
        """Cria um novo pedido com um número máximo de tentativas.
         Depois disso, o pedido será considerado uma falha."""

        self.deadline = None
        self.status = Status()
        self.timeout = timeout
        self.tries = tries

        # Mensagem original do rest
        #self.msg = message

        # Separa o path e o conteúdo do pedido
        self.extract_path(message)

    def extract_path(self, message):
        """Separa informações de roteamento e dados de um pedido recebido."""
        self.__create_pack(message)
        # Verifica se o pacote de configurações é válido
        if self.pack.ready():
            self.id = self.pack.id
            self.service_id = self.pack.service_id
            self.path = [str(self.id), '']

            #print "self.content=============================>>>>>", self.content

    def update_deadline(self):
        """Atualize o prazo de solicitação se um tempo limite tiver sido especificado."""
        if self.timeout is not None:
            self.deadline = time.time() + self.timeout
        if self.tries is not None:
            self.tries -= 1

    def make_request(self):
        """Devolve um pedido por sua identificação exclusiva para ser enviada a um worker.
         Isso também atualiza o prazo de solicitação."""
        self.update_deadline()

        # self.path[0] = self.id
        self.message = self.path + self.content

    def expired(self):
        """Se verdadeiro o número máximo de tentativas foi realizado."""
        return self.tries == 0

    def timed_out(self):
        """Se verdadeiro passamos o prazo de execução do pedido."""
        return self.deadline is not None and time.time() > self.deadline

    # --------------------------------
    # __create_pack
    # --------------------------------
    def __create_pack(self, message):
        # Verifica se a mensagem é uma lista
        # Verifica se há 3 elementos (multipart)
        if type(message) == list and len(message) == 3:
            # Obtém a string JSON do pacote
            self.content = message[2:]
            # Verifica se o conteúdo da mensagem é uma lista
            # Verifica se a lista contém um elemento
            if type(self.content) == list and len(self.content) == 1:
                # Prepara configurações recebidas do REST
                self.pack = Pack(self.content[0], self.status)
            else:
                self.status.error("ERROR_STACK", None, ["__pack de dados vindo do broker nao recebeu conteudo valido"])
        else:
            self.status.error("ERROR_STACK", None, ["__pack de dados vindo do broker nao e valido"])

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.status.ready() and self.pack.ready()

    # --------------------------------
    # send_error
    # --------------------------------
    def send_error(self):
        return self.path + self.pack.send_error()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='packagetest108',
      version='0.108',
      description='DevDoo Microservices',
      url='http://github.com/storborg/funniest',
      author='Israel Agoeiro',
      author_email='israelagoeiro@gmail.com',
      license='MIT',
      packages=['devdoo', 'devdoo/yaml'],
      zip_safe=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from bson.json_util import dumps
from datafilter import DataFilter
from convert import Convert
from pack import Pack
from pprint import pprint

# TODO:: Refatorar completamente a classe DataRest
# TODO:: Implementar configuração de limites definido na console para o serviço
class DataRest:

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, message, status, is_database=False):
        self.status = status
        # print "[[[[[[[[[[MESSAGE]]]]]]]]]]]]", message
        self.pack = Pack(message, status)

        # self.pack.show('PACK')

        # Variáveis default
        self.__collection = None
        self.__data = dict()
        self.__database_id = None
        self.__default_limit = 100
        self.__fields = []
        self.__filter = None
        self.__limit = 0
        self.__max_limit = 200
        self.__method = None
        self.__offset = 0
        self.__regex = None
        self.__result_type = None
        self.__search = None
        self.__sort = None
        self.__public_access = True
        self.__message_send = None

        # TODO:: Aprimorar ready
        if self.pack.ready():
            if is_database:
                self.decode_database()
            else:
                self.decode_service()
        else:
            self.pack.show()

    # --------------------------------
    # action
    # --------------------------------
    def action(self, action):
        self.pack.action = action

    def action_db(self):
        return self.pack.action

    # --------------------------------
    # body
    # --------------------------------
    def body(self):
        return self.__body

    # --------------------------------
    # collection
    # --------------------------------
    def collection(self, collection):
        self.__collection = collection

    def get_collection(self):
        return self.__collection


    def data(self, data):
        self.__data = data

    def get_data(self):
        return self.__data

    # --------------------------------
    # database
    # --------------------------------
    def database(self):
        return self.pack.database

    # --------------------------------
    # decode_database
    # --------------------------------
    def decode_database(self):
        body = self.pack.body
        self.__data = body["data"]
        self.__collection = body["collection"]
        self.__fields = body["fields"]
        self.__filter = body["filter"]
        self.__limit = int(body["limit"])
        self.__method = body["method"]
        self.__offset = int(body["offset"])
        self.__regex = body["regex"]
        self.__result_type = body["result_type"]
        self.__search = body["search"]
        self.__sort = body["sort"]

    # --------------------------------
    # decode_service
    # --------------------------------
    def decode_service(self):
        body = self.pack.body
        self.__body = body["data"]
        self.__fields = body["fields"]
        self.__filter = DataFilter(body["filter"], self.status)
        self.__limit = int(body["limit"])
        self.__method = body["method"]
        self.__offset = int(body["offset"])
        self.__regex = body["regex"]
        self.__search = body["search"]
        self.__sort = body["sort"]

    # --------------------------------
    # fields
    # --------------------------------
    def fields(self, fields):
        # Verifica se recebeu uma lista de campos do usuário
        # Verifica se campos são válidos
        # Registra lista de campos solicitados
        if len(self.__fields) > 0:
            list_fields = []
            for item in self.__fields:
                if item in fields:
                    list_fields.append(item)
            self.__fields = list_fields
        else:
            # Registra lista de campos default
            self.__fields = fields

    # --------------------------------
    # filter
    # --------------------------------
    def filter(self):
        return self.__filter

    # --------------------------------
    # info
    # --------------------------------
    def info(self):
        return {
            "owner": {
                "id": self.__token,
                "last_id": self.__token
            }
        }

    # --------------------------------
    # limit
    # --------------------------------
    def get_limit(self):
        return self.__limit

    # --------------------------------
    # offset
    # --------------------------------
    def offset(self):
        return self.__offset

    # --------------------------------
    # get_fields
    # --------------------------------
    def get_fields(self):
        return self.__fields

    # --------------------------------
    # limit
    # --------------------------------
    # Define o limite de documentos retornados
    #
    # ?limit=20
    #
    def limit(self, max_limit=0):
        # Condigura o limite default do serviço
        limit = self.__default_limit

        # Verifica se o limit da ação foi definido
        if self.__limit > 0:
            # Recupera o limite da operação para validar
            limit = error_limit = self.__limit

            # Verifica se o limite da operação maior do que o maximo permitido pelo serviço
            if limit > self.__max_limit:

                # Caso seja, então registra o limite maximo permitido
                limit = error_max_limit = self.__max_limit

                # Verifica se um limite máximo foi definido pelo desenvolvedor do serviço
                if (max_limit > 0) and (limit > max_limit):
                    # Caso seja, então registra o limite maximo permitido
                    error_max_limit = max_limit
                    limit = max_limit

                # Registra mensagem de erro no serviço de log
                self.status.warn("MAX_LIMIT_EXCEEDED", None, [Convert.to_str(error_max_limit), Convert.to_str(error_limit)])

        self.__limit = limit

    # --------------------------------
    # sort
    # --------------------------------
    def sort(self):
        return self.__sort

    # --------------------------------
    # method
    # --------------------------------
    def method(self):
        return self.__method

    # --------------------------------
    # public_access
    # --------------------------------
    # TODO:: Implementar verificação de usuário logado
    def public_access(self):
        return self.__public_access

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.pack.ready()

    # --------------------------------
    # result
    # --------------------------------
    def result(self, message):
        message["elapsed_time"] = {"response": "17.000", "server": "7.000"}
        message["method"] = self.__method
        if self.status.has_error():
            message["errors"] = self.status.to_list()
        self.__message_send = message


    # --------------------------------
    # send_result
    # --------------------------------
    # Registrar no stack de log
    # Registrar no stack de analytic
    # Registrar no stack de error
    # Registrar no stack de history
    # self.analyric.add(dict())
    def send_result(self):
        message = {
            "action": self.pack.action,
            "active_port": self.pack.active_port,
            "alerts": {
                "error": self.status.to_list(),
                "log": None,
                "warn": None,
                "info": None,
            },
            "app_id": self.pack.app_id,
            "api_key": self.pack.api_key,
            "body": self.__message_send,
            "id": self.pack.id,
            "length_in": self.pack.length_in,
            "length_out": None,
            "service": self.pack.service,
            "service_id": self.pack.service_id,
            "database_id": self.pack.database_id,
            "open": False,
            "source": self.pack.source,
            "success": True,
            "time": {
                "time_start": None,
                "service": None,
                "database": None
            }
        }
        return [dumps(message)]

    # --------------------------------
    # result_type
    # --------------------------------
    def result_type(self, result_type):
        self.__result_type = result_type

    # --------------------------------
    # get_result_type
    # --------------------------------
    def get_result_type(self):
        return self.__result_type

    # --------------------------------
    # show
    # --------------------------------
    # Imprime propriedade/valor do objeto
    def show(self, message=None):
        if message:
            print message, ":::------->>>"
        pprint(self.__dict__)

    # --------------------------------
    # send_database
    # --------------------------------
    # TODO:: Implementar melhorias na verificação de disponibilidade do serviço para serguir adiante, chegando todos os tipos que serão enviados
    # TODO:: Verificar se os dados para serem enviados ao serviço de base de dados estão dentro das especificações esperadas
    def send_database(self):
        message = {
            "action": self.pack.action,
            "active_port": self.pack.active_port,
            "alerts": {
                "error": self.status.to_list(),
                "log": None,
                "warn": None,
                "info": None,
            },
            "app_id": self.pack.app_id,
            "api_key": self.pack.api_key,
            "body": {
                "collection": self.__collection,
                "data": self.__data,
                "fields": self.__fields,
                "filter": self.__filter.to_list(), #TODO:: Implementar filter
                "limit": self.__limit,
                "offset": self.__offset,
                "method": self.__method,
                "result_type": self.__result_type,
                "regex": self.__regex,
                "search": self.__search,
                "sort": self.__sort
            },
            "id": self.pack.id,
            "length_in": self.pack.length_in,
            "length_out": None,
            "service": self.pack.service,
            "service_id": self.pack.database_id,
            "database_id": self.pack.database_id,
            "open": False,
            "source": self.pack.source,
            "success": False,
            "time": {
                "time_start": None,
                "service": None,
                "database": None
            }
        }

        #print "SEND_DATABASE----->>>"
        # pprint(message)

        # Montar mensagem que será enviada para o serviço de base de dados
        return dumps(message)

    # --------------------------------
    # send_service_error
    # --------------------------------
    # Montar mensagem de erro para ser retornada ao servidor rest
    # TODO: Preparar filtro para ser retornado ao cliente
    def send_service_error(self):
        message = {
            "action": self.pack.action,
            "active_port": self.pack.active_port,
            "alerts": {
                "error": self.status.to_list(),
                "log": None,
                "warn": None,
                "info": None,
            },
            "app_id": self.pack.app_id,
            "api_key": self.pack.api_key,
            "body": {
                "success": False,
                "message": "Falha de interação com microserviço",
                "elapsed_time": {"response": "17.000", "server": "7.000"},
                "query": {
                    "limit": self.__limit,
                    "offset": self.__offset,
                    "fields": self.__fields,
                    "filter": self.__filter.to_list()
                },
            },
            "id": self.pack.id,
            "length_in": self.pack.length_in,
            "length_out": None,
            "service": self.pack.service,
            "service_id": self.pack.service_id,
            "database_id": None,
            "open": False,
            "source": self.pack.source,
            "success": False,
            "time": {
                "time_start": None,
                "service": None,
                "database": None
            }
        }

        #print "SEND_SERVICE_ERROR", "----->>>>"
        #pprint(message)

        return dumps(message)

    # --------------------------------
    # source
    # --------------------------------
    def source(self):
        return self.pack.source

    # --------------------------------
    # validate_service
    # --------------------------------
    # Testa a validade do serviço
    #
    def validate_service(self, service, service_id):
        service = Convert.to_str(service)
        service_id = Convert.to_str(service_id)
        if (self.pack.service != service) or (self.pack.service_id != service_id):
            # Registra mensagem de erro no serviço de log
            self.status.error("SERVICE_FAILURE", None, [self.pack.service, service, self.pack.service_id, service_id])
            return False
        return True

    # --------------------------------
    # __filter_extras
    # --------------------------------
    # TODO:: Finalizar sistema de seach e regex
    def __filter_extras(self):
        # self.__prepare_regex()
        self.__prepare_search()

    # --------------------------------
    # __prepare_regex
    # --------------------------------
    def __prepare_regex(self):
        print self.__regex

    # --------------------------------
    # __prepare_search
    # --------------------------------
    # { name: { $regex: "s3", $options: "si" } }
    # or{quantity.lt(20)|price(10)}
    # tags.in( '{ $regex: "s3", $options: "si" }', '{ $regex: "s3", $options: "si" }')
    # TODO:: Implementar prepare serach
    def __prepare_search(self):
        values, fields = self.__search.split(';')
        values = values.split()
        fields = fields.split(',')
        if len(fields) > 0:
            for item in fields:
                for word in values:
                    # self.__filter.add("_id.eq", result.group(2))
                    print "------->", item, word

    # --------------------------------
    # __set_source
    # --------------------------------
    # Recupera o id do documento
    '''
    def __set_source(self, source):
        # Verificar se o source possui uma id
        # Pegar a id do source
        result = re.search(r"([\s\S]+)([0-9a-f]{24})$", source)
        if result:
            source = result.group(1) + ":id"
            self.__filter.add("_id.eq", result.group(2))
        return source
    '''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from broker import Broker

# --------------------------------
# Brokers
# --------------------------------
'''
Gerencia a lista de brokers utilizada pelo serviço
'''
class Brokers:

    '''
    Inicializa a classe

    :param service_port: Porta do serviço
    :param brokers: Objeto de broker para registrar
    :param status: Gerenciador de erros
    :param has_connection_backend:  Status indica se deve configurar endpoind para conexão backend
    :param has_connect_frontend: Status indica se deve configurar endpoind para conexão frontend
    '''
    def __init__(self, service_port, brokers, status, has_connection_backend=False, has_connect_frontend=False):
        # Variaveis privadas
        self.__brokers = []

        # Variaveis parametros
        self.brokers = brokers
        self.has_connection_backend = has_connection_backend
        self.has_connect_frontend = has_connect_frontend
        self.status = status
        self.service_port = service_port

        self.__config()

    # --------------------------------
    # available
    # --------------------------------
    '''
    Recupera o broker disponível para atender o serviço
    '''
    def available(self):
        return self.__brokers[0]

    # --------------------------------
    # __config
    # --------------------------------
    # Registra brokers na lista de brokers
    def __config(self):
        if len(self.brokers) > 0:
            for broker_id in self.brokers:
                self.__brokers.append(
                    Broker(broker_id, self.status, self.service_port, self.has_connection_backend, self.has_connect_frontend))
        else:
            self.status.error("ERROR_BROKER_CONFIG", None, ["Nao recebeu lista de brokers para configurar no servico"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
from convert import Convert

class Services:

    def __init__(self, services, status):
        # Status indica se o servico está pronto
        self.ready = False
        self.status = status

        self.__services = dict()
        self.__config(services)

    # --------------------------------
    # __config
    # --------------------------------
    # Registra servicos na lista de servicos
    def __config(self, services):
        if len(services) > 0:
            from service import Service
            for item in services:
                service = Service(item, self.status)
                # Verifica se a configuracao é válida
                if (service is not None) and service.ready():
                    # Registra o servico no stack de servicos
                    self.__services[service.id] = service
                else:
                    self.status.error("INVALID_SERVICE", None, ["FALHA NA INICIALIZACAO DO SERVICO", service.id])
        else:
            self.status.error("ERROR_SERVICE_CONFIG", None, ["Nao recebeu lista de servico para configurar no broker"])

    # --------------------------------
    # find
    # --------------------------------
    # Localiza os servicos
    def find(self, service_id):
        if service_id in self.__services.keys():
            return self.__services[service_id]
        self.status.error("INVALID_SERVICE", None, ["Servico nao configurado", service_id])
        return None

    # --------------------------------
    # ready
    # --------------------------------
    # Retorna status indicando se houve algum erro no processo de configuracao dos servicos e se existe servicos para ser executado
    def ready(self):
        return self.status.ready() and (len(self.__services.keys()) > 0)

    # --------------------------------
    # __has_services
    # --------------------------------
    def __has_services(self, list_ids):
        error = False
        if (type(list_ids) == list) and (len(list_ids) > 0):
            for item in list_ids:
                if not re.match("^[0-9a-f]{24}$", Convert.to_str(item)):
                    error = True
        if error is True:
            self.status.error("INVALID_SERVICE", None, ["Lista de Servicos nao valida", list_ids])

        return self.status.ready()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from connection import Connection
from services import Services
from pprint import pprint
from socket import Socket

# --------------------------------
# Broker
# --------------------------------
'''
A classe Broker gerencia configurações do broker
'''

class Broker:

    '''
    Inicializa a classe

    :param broker_id: Identificador do broker
    :param status: Gerenciador de erros
    :param service_port: Porta do serviço
    :param has_connection_backend:  Status indica se deve configurar endpoind para conexão backend
    :param has_connect_frontend: Status indica se deve configurar endpoind para conexão frontend
    '''
    def __init__(self, broker_id, status, service_port=None, has_connection_backend=False, has_connect_frontend=False):
        # Variaveis privadas
        self.__sockets = dict()

        # Variaveis default
        self.network = None
        self.services = None
        self.type = None
        self.connect_backend = None
        self.connect_frontend = None

        # Variaveis parametros
        self.id = broker_id
        self.status = status
        self.has_connection_backend = has_connection_backend
        self.has_connect_frontend = has_connect_frontend
        self.service_port = service_port

        print ('BROKER_ID =====>', broker_id)

        # Variaveis inicializadas
        self.connection = Connection(status)

        # Verifica se o arquivo de configuração foi inicializado corretamente
        if self.__has_config() is not True:
            self.status.error("INVALID_SERVICE", None, ["FALHA NA INICIALIZACAO DE BROKER", broker_id])
            self.status.to_print()
        else:
            self.status.show("BROKER_INIT", [self.id, self.type])
            # self.show("BROKER")

    # --------------------------------
    # __setitem__
    # --------------------------------
    '''
    Configura a classe para criar propriedades dinamicamente 
    '''
    def __setitem__(self, field, value):
        self.field = value

    # --------------------------------
    # ready
    # --------------------------------
    '''
    Status indica se está pronto
    :return: void
    '''
    def ready(self):
        return self.status.ready()

    # --------------------------------
    # show
    # --------------------------------
    '''
    Imprime propriedade/valor do objeto

    :param title: Título identificador
    :return: void
    '''
    def show(self, title=None):
        if title is not None:
            print title, ":::------->>>"
        pprint(self.__dict__)

    # --------------------------------
    # __bind
    # --------------------------------
    '''
    Configura endpoint de conexão backend
    Configura endpoint de conexão frontend
    Configura endpoint de conexão bind
    '''
    def __bind(self, service_port):
        # Verifica se uma rede foi definida e se a porta do serviço não foi fornecida
        if (self.network is not None) and (service_port is None):
            self.bind_frontend = "tcp://*:" + str(self.network["port"])

        # Verifica se uma rede foi definida e se a porta do serviço foi fornecida
        elif self.network and service_port:
            # Verifica se deve configurar um endpoint de conexão backend
            if self.has_connection_backend:
                self.connect_backend = "tcp://" + self.network["ip"] + ":" + str(service_port)

            # Caso contrário verifica se deve configurar um endpoint de conexão frontend
            elif self.has_connect_frontend:
                self.connect_frontend = "tcp://" + self.network["ip"] + ":" + str(self.network["port"])

            # Caso contrário configura um endpoint de conexão bind
            else:
                self.bind_backend = "tcp://*:" + str(service_port)
        else:
            self.status.error("ERROR_BROKER_CONFIG", None, ["Nao obteve dados de configuracao do network no broker", self.id])

    # --------------------------------
    # __config_services
    # --------------------------------
    '''
    Implementa configurações de serviços filhos do broker
    '''
    def __config_services(self):
        # Verifica se foi definido uma porta de serviço e se a lista de serviços existe
        if (self.service_port is None) and (self.services is not None):
            # Implementa lista de serviços
            self.services = Services(self.services, self.status)
            # Cria endpoints de conexões disponiveis para o broker
            self.__bind(service_port=None)
        # Caso contrário verifica se a porta foi fornecida
        elif self.service_port is not None:
            self.__bind(self.service_port)
        else:
            self.status.error("ERROR_BROKER_CONFIG", None, ["Nao obteve dados de configuracao dos servicos filhos do broker", self.id])

    # --------------------------------
    # __has_config
    # --------------------------------
    '''
    Implementa auto configuração do serviço
    Vai até o servidor de configuração buscar informações de configuração
    '''
    def __has_config(self):
        if self.id:
            # Recupera configurações no servidor de serviços API
            data_response = self.connection.load_config(self.id)

            # Verifica se a resposta do servidor é válida
            if data_response.ready():
                # Registra na classe as propriedades da configuração
                data = data_response.values()
                for field in data:
                    self.__dict__[field] = data[field]

                # Configura serviços-filhos
                self.__config_services()
            else:
                self.status.error("ERROR_BROKER_CONFIG", None, ["Nao obteve dados de configuracao do broker", self.id])
        else:
            self.status.error("ERROR_BROKER_CONFIG", None, ["Nao recebeu identificador de broker", "--"])

        # Retorna se o status está pronto
        return self.status.ready()

    # --------------------------------
    # socket
    # --------------------------------
    '''
    Retorna o socket do serviço
    '''
    def socket(self, stack):
        socket = None
        if stack.ready():
            # Recupera o serviço
            service = self.services.find(stack.service_id)

            # Verifica se o serviço existe e possui um endpoint de backend na lista de sockets
            if (service is not None) and service.endpoint_backend() in self.__sockets.keys():
                socket = self.__sockets[service.endpoint_backend()]
            # Verifica se o serviço existe e o pacote é válido
            elif (service is not None) and stack.pack.is_valid():
                # Gera um novo Socket
                socket = Socket(service)
                # Verifica se o socket está pronto
                if socket.ready():
                    # Adiciona o novo socket na lista de sockets
                    self.__sockets[service.endpoint_backend()] = socket
                else:
                    # Gera um socket vazio
                    socket = Socket(None)
            else:
                # Gera um socket vazio
                socket = Socket(None)

            # Verifica se o socket não está pronto e adiciona mensagem de erro
            if socket.ready() is not True:
                stack.status.error("INVALID_SERVICE", None, ["Servico nao disponivel no servidor BROKER", stack.pack.service_id])

        # Retorna o Socket gerado
        return socket



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq, json
from service_db import ServiceDB
from service import Service
from yml import Yml
from status import Status
from pymongo import MongoClient

# Conecta ao banco de dados
db_host = 'database.unicoode.com'
db_username = 'devdoo'
db_password = 'mudar123'
db_database = 'admin'
db_port = 27017
mongo_server = MongoClient(
    db_host,
    username=db_username,
    password=db_password,
    authSource=db_database,
    authMechanism='SCRAM-SHA-1',
    port=db_port)
#mongo_server = MongoClient(db_host, db_port)

# TODO:: Enviar para o servidor console a informação de falha de configuração
class DevdooDatabase(object):

    # --------------------------------
    # Error
    # --------------------------------
    # TODO:: Remover classe de error
    class Error(Exception):
        pass

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self):
        super(DevdooDatabase, self).__init__()

        # Variaveis default
        self.id = None
        self.service = None
        self.serviceDB = None
        self.zmq_socket_broker = None

        # Variaveis inicializadas
        self.status = Status()

        # Verifica se o arquivo de configuração foi inicalizado corretamente
        if self.__has_config():
            self.__init()
        else:
            self.status.error("INVALID_SERVICE", None, ["FALHA NA INICIALIZACAO DO SERVICO", "DEVDOO-DATABASE"])
            self.status.to_print()

    # --------------------------------
    # __has_config
    # --------------------------------
    '''
    Obtem configurações do servico

    :return: boolean
    '''
    def __has_config(self):
        # Recupera o identificador do servico
        yml = Yml(self.status)
        self.id = yml.id()

        # Verifica se existe um identificador do servico
        if self.status.ready() and self.id:
            self.service = Service(self.id, self.status, True)

        # Retorna o status do serviço de banco
        return self.status.ready() and self.service and self.service.ready()

    # --------------------------------
    # __init
    # --------------------------------
    '''
    Inicializa o serviço

    :return: void
    '''
    def __init(self):
        # Cria contexto ZMQ
        context = zmq.Context()

        # Adaptador de conexão com o broker
        self.zmq_socket_broker = context.socket(zmq.REP)

        # Conecta ao broker secundário
        self.zmq_socket_broker.connect(self.service.broker_connect_backend())

        # Executa o loop da operação
        self.run()

    # --------------------------------
    # operation
    # --------------------------------
    # Executa a ação do Serviço
    #
    # TODO:: Implementar controle ready antes de seguir adiante
    def operation(self):
        # Recebe a mensagem do broker secundário
                 #self.zmq_socket_broker.recv()
        message = self.zmq_socket_broker.recv_multipart()
        self.serviceDB = ServiceDB(message, mongo_server)

        print "RECEBEU DO REST------->>>>", type(message), message
        # print "\nRECEBEU DO REST BBBB------->>>>", type(self.serviceDB.send_result()), self.serviceDB.send_result()

        # Envia mensagem de retorno para o cliente
        self.zmq_socket_broker.send_multipart(self.serviceDB.send_result())

    # --------------------------------
    # operation
    # --------------------------------
    # Executa a ação do Serviço
    #
    def operationXXX(self):
        # self.serviceDB = ServiceDB(self.zmq_socket_broker.recv(), self.config)
        print "self.service =========>>> ", self.service

        # Recebe a mensagem do broker secundário
        message = self.zmq_socket_broker.recv_multipart()
        # print "\nRECEBEU DO REST------->>>>", type(message), message

        # Converte a mensagem em um JSON válido
        valid_json = json.dumps(message[0])
        # print "\nCONVERTED ---->>>>", type(valid_json), valid_json

        # Converte o JSON em dicionário
        message_json = json.loads(json.loads(valid_json))
        # print "\nMESSAGE JSON ---->>>>", type(message_json), message_json

        # Processa a operacao
        # self.zmq_socket_broker.send(self.serviceDB.send())

        # Preenche o campo data (exemplo)
        message_json["body"]["data"] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"

        # Gera lista de retorno e adiciona a mensagem
        message = list()
        message.append(json.dumps(message_json))
        print "\nRETURN MESSAGE =====>", type(message), message

        # Envia mensagem de retorno para o cliente
        self.zmq_socket_broker.send_multipart(message)
        # print "ENVIA DE VOLTA AO REST<<<<<-------------------", message

    # --------------------------------
    # run
    # --------------------------------
    # Executa a operação em um loop infinito
    #
    # TODO:: Implementar ping de verificação de disponibilidade do serviço
    def run(self):
        while True:
            try:
                self.operation()
            except DevdooDatabase.Error:
                # except Exception as inst:
                #    print "API_DB::RUN::ERROR", inst.args
                self.status.error("SERVICE_FAILURE", None, ["FALHA NA EXECUCAO DO SERVICO", "DEVDOO-DATABASE"])
                self.status.to_print()
